/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillaengine.h: Mozilla Web Engine Interface Declaration
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 09:35:49 -0400 (Wed, 02 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include <gwebengine.h>

#include <gtkmozembed.h>
#include <gtkmozembed_internal.h>
#include <gtkmozembedprivate.h>

#include "prenv.h"
#include "nsMemory.h"

#ifndef __GMOZILLAENGINE_H__
#define __GMOZILLAENGINE_H__

G_BEGIN_DECLS

#define G_TYPE_MOZILLA_ENGINE             (g_mozilla_engine_get_type ())
#define G_MOZILLA_ENGINE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_TYPE_MOZILLA_ENGINE, GMozillaEngine))
#define G_MOZILLA_ENGINE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), G_TYPE_MOZILLA_ENGINE, GMozillaEngineClass))
#define G_IS_MOZILLA_ENGINE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_TYPE_MOZILLA_ENGINE))
#define G_IS_MOZILLA_ENGINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_TYPE_MOZILLA_ENGINE))
#define G_MOZILLA_ENGINE_GET_CLASS(iobj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_TYPE_MOZILLA_ENGINE, GMozillaEngineClass))

enum {
    TITLE_CHANGED,
    URL_CHANGED,
    LOADING_PROGRESS,
    START_LOADING,
    FINISHED_LOADING,
    ENCODING,
    DOWNLOAD_REQUEST,
    AUTHENTIFICATION_REQUIRED,
    ALERT,
    CONFIRM,
    PROMPT,
    MAILTO,
    UNKNOWN_PROTOCOL,
    SECURITY_MODE_CHANGED,
    WINDOW_REQUEST,
    CLOSE_WINDOW,
    RAISE_REQUEST,
    LOWER_REQUEST,
    UPLOAD_DIALOG,
    FILE_UPLOAD_STARTED,
    FILE_UPLOAD_PROGRESS,
    FILE_UPLOAD_FINISHED,
    WEBENGINE_POPUP_MENU,
    EXTERNAL_ACTION,
    MODAL_DIALOG,
    FRAME_STACKING_CHANGED,
    MODAL_CERTIFICATE_DIALOG,
    ASK_COOKIE,
    CERTIFICATE_DIALOG,
    CERTIFICATE_PASSWORD_DIALOG,
    CERTIFICATE_DETAILS,
    LAST_SIGNAL
};

typedef struct _GMozillaEngine GMozillaEngine;
typedef struct _GMozillaEngineClass GMozillaEngineClass;
typedef struct _GContextData GContextData;

struct _GContextData
{
    gchar *imghref, *href, *linktext, *framehref;
    gint xcoord, ycoord;
    glong ctx_type;
};

struct _GMozillaEngine
{
    //	GObject parent;
    GtkMozEmbed parent;
    GObject* engine;

    gint timer_id;
    gint cur_requests;
    gint total_requests;
    GContextData ctx_data;
};

struct _GMozillaEngineClass
{
    GtkMozEmbedClass parent_class;

    void (*title)	(GMozillaEngine *embed);
    void (*net_stop)	(GMozillaEngine *embed);
    void (*net_start)	(GMozillaEngine *embed);
    void (*location)	(GMozillaEngine *embed);
    void (*progress)	(GMozillaEngine *embed, gint curprogress, gint maxprogress);
    void (*net_state)   (GMozillaEngine *embed, gint flags, guint status);
    void (*net_state_all)   (GMozillaEngine *embed, gint flags, guint status);

    // engine new signals
    void (* url_changed)		(GMozillaEngine *embed, char *);
    void (* close_window)		(GMozillaEngine *embed);;
    //void (* title)              	(GMozillaEngine *embed, char *);
    void (* start_loading)      	(GMozillaEngine *embed, const char *);
    void (* finished_loading)		(GMozillaEngine *embed, int , int , int);
    void (* window_request)     	(GMozillaEngine *embed, gpointer , char *, PRBool);
    void (* loading_progress)   	(GMozillaEngine *embed, gint curprogress, gint maxprogress);
    gint (* encoding) 	        	(GMozillaEngine *embed, uint encoding);
    void (* security_mode_changed)	(GMozillaEngine *embed, gpointer request, guint state);
    gint (* download_request)    	(GMozillaEngine *embed, const gchar *, const gchar *, const gchar *, glong, gint);
    gint (* authentification_required)  (GMozillaEngine *, gint , gchar *);
    gint (* alert)    			(GMozillaEngine *, const gchar *);
    gboolean (* confirm)		(GMozillaEngine *, const gchar *);
    gchar*(* prompt)   			(GMozillaEngine *, const gchar *, const gchar *);
    gint (* mailto)    			(GMozillaEngine *, const gchar *);
    void (* unknown_protocol)    	(GMozillaEngine *, const gchar *);
    void (* raise_request)    		(GMozillaEngine *);
    void (* lower_request)    		(GMozillaEngine *);
    gboolean (* upload_dialog)   	(GMozillaEngine *, const gchar *, const gchar *, gpointer );
    void (* file_upload_started)	(GMozillaEngine *);
    void (* file_upload_progress)	(GMozillaEngine *, gint , const gchar *);
    void (* file_upload_finished)	(GMozillaEngine *);
    void (* webengine_popup_menu)	(GMozillaEngine *, uint , gint, int , gchar *, gchar *, gchar *, gchar *);
    void (* external_action)		(GMozillaEngine *, const gchar *);
    gint (* modal_dialog)		(GMozillaEngine *, const gchar *, const gchar *, gint, gint, gint, gint);
    void (* frame_stacking_changed)	(GMozillaEngine *, gboolean);
    gint (* modal_certificate_dialog) 	(GMozillaEngine *, const gchar *, gint, gint, gint, gint, gint);

    // web signals 
    void (* ask_cookie)			(GObject * , gpointer , gint , const gchar * , const gchar * ,
                   			const gchar * , const gchar *, const gchar * , gboolean , gint , GObject *);
    void (* certificate_dialog) 	(GObject * , GObject *);
    void (*certificate_password_dialog) (GObject *, const gchar *, const gchar *, gchar **);
    void (*certificate_details)		(GObject *, gpointer );

};

GType g_mozilla_engine_get_type (void);

static GMozillaEngine *cur_engine;

GObject* g_mozilla_engine_new                         (void);
GObject* g_mozilla_engine_new_with_context            (gpointer context);
GObject* g_mozilla_engine_new_with_context_and_tracks (gpointer context,
        gboolean leave_tracks);

GMozillaEngine* g_mozilla_get_current_engine(void);
gint on_button_pressed_cb(GtkWidget *, gpointer *, GMozillaEngine *);
gint on_button_released_cb(GtkWidget *, gpointer *, GMozillaEngine *);

G_END_DECLS

#endif /* __GMOZILLAENGINE_H__ */
