/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillatransferitem.h: Mozilla Transfer Item Interface Declaration
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 15:35:49 +0200 (Wed, 02 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include <gwebtransferitem.h>
#include <nsIExternalHelperAppService.h>
#include <nsIRequest.h>

#ifndef __GMOZILLATRANSFERITEM_H__
#define __GMOZILLATRANSFERITEM_H__

G_BEGIN_DECLS

#define G_TYPE_MOZILLA_TRANSFER_ITEM             (g_mozilla_transfer_item_get_type ())
#define G_MOZILLA_TRANSFER_ITEM(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_TYPE_MOZILLA_TRANSFER_ITEM, GMozillaTransferItem))
#define G_MOZILLA_TRANSFER_ITEM_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST ((vtable), G_TYPE_MOZILLA_TRANSFER_ITEM, GMozillaTransferItemClass))
#define G_IS_MOZILLA_TRANSFER_ITEM(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_TYPE_MOZILLA_TRANSFER_ITEM))
#define G_IS_MOZILLA_TRANSFER_ITEM_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE ((vtable), G_TYPE_MOZILLA_TRANSFER_ITEM))
#define G_MOZILLA_TRANSFER_ITEM_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_INTERFACE ((inst), G_TYPE_MOZILLA_TRANSFER_ITEM, GMozillaTransferItemClass))

/* a macro for creating a new object of our type */
#define GET_NEW (g_object_new(g_mozilla_transfer_item_get_type(), NULL))

typedef struct _GMozillaTransferItem GMozillaTransferItem;
typedef struct _GMozillaTransferItemClass GMozillaTransferItemClass;
typedef struct _TransferItem TransferItem;

/*
struct _GWebTransferItem
{
	GObject parent;
 
	const gchar*  		filename; 
	const gchar*  		filetype; 
	glong         		filesize; 
	const gchar*  		url; 
	const gchar*  		handler_app; 
	glong         		downloaded_size; 
	gboolean      		isPaused;
	gboolean 			open_with; 
};
*/

struct _GMozillaTransferItem
{
    GObject parent;
    GObject* item;

    const gchar* 			file_name;
    const gchar* 			server;
    glong					file_size;
    const gchar* 			file_type;
    const gchar*			handler_app;
    glong         			downloaded_size;
    GWebTransferItem* 		transferItem;
    gboolean      			isPaused;
    gboolean 				open_with;
    gboolean				started;
    nsIHelperAppLauncher*	launcher;
    nsIRequest* 			request;
};

struct _GMozillaTransferItemClass
{
    GObjectClass parent;

    void (*started) (GMozillaTransferItem* item);
    void (*completed) (GMozillaTransferItem* item);
    void (*error) (GMozillaTransferItem* item);
    void (*aborted) (GMozillaTransferItem* item);
    void (*progress) (GMozillaTransferItem* item, gulong downloaded_bytes, gulong total_bytes, gdouble kbps);
};

GType g_mozilla_transfer_item_get_type (void);

GObject* g_mozilla_transfer_item_new           (void);
GObject* g_mozilla_transfer_item_new_with_item (GObject *item);
GObject* g_mozilla_transfer_item_new_with_url  (const gchar *url,
        const gchar *filename);

static GObject* g_mozilla_transfer_item_get_web_item (GMozillaTransferItem *);

enum
{
    TRANSFER_ITEM_STARTED_SIGNAL,
    TRANSFER_ITEM_COMPLETED_SIGNAL,
    TRANSFER_ITEM_ERROR_SIGNAL,
    TRANSFER_ITEM_ABORTED_SIGNAL,
    TRANSFER_ITEM_PROGRESS_SIGNAL,
    TRANSFER_ITEM_LAST_SIGNAL
};

static guint transfer_item_signals[TRANSFER_ITEM_LAST_SIGNAL] = { 0 };

G_END_DECLS

#endif /* __GMOZILLATRANSFERITEM_H__ */
