#include "gmozillacontenthandler.h"
#include "gmozillaengine.h"
#include "gmozillatransferitem.h"

#include <nsIPromptService.h>
#include <nsIWebProgressListener2.h>
#include <nsCOMPtr.h>
#include <nsIServiceManager.h>
#include <nsIInterfaceRequestorUtils.h>
#include <nsIURI.h>
#include <nsIURL.h>
#include <nsIMIMEInfo.h>
#include <nsILocalFile.h>
#include <nsIDOMWindow.h>
#include <nsIExternalHelperAppService.h>
#include <nsCExternalHandlerService.h>
#include <nsMemory.h>
#include <nsNetError.h>
#include <nsEmbedString.h>
#include <nsIStreamListener.h>
#include <nsIFile.h>
#include <nsILocalFile.h>


#include <unistd.h>

class ContentHandler;

GMozillaTransferItem *gTransferItemTemp;

class ProgressListener : public nsIWebProgressListener2
{
public:
    ProgressListener(GMozillaTransferItem *aTransferItem)
    {
        mTransferItem = aTransferItem;
    };
    ~ProgressListener()
    {}
    ;

    NS_DECL_ISUPPORTS
    NS_DECL_NSIWEBPROGRESSLISTENER
    NS_DECL_NSIWEBPROGRESSLISTENER2

    GMozillaTransferItem *mTransferItem;
};

NS_IMPL_ISUPPORTS2(ProgressListener, nsIWebProgressListener2, nsIWebProgressListener)
NS_IMPL_ISUPPORTS1(ContentHandler, nsIHelperAppLauncherDialog)

ContentHandler::ContentHandler()
{
    //printf("ContentHandler::ContentHandler constructor(%p)\n", this );
}

ContentHandler::~ContentHandler()
{
    //printf("ContentHandler::ContentHandler destructor(%p)\n", this);
}

// where everything in UI starts
NS_IMETHODIMP ContentHandler::Show(nsIHelperAppLauncher *aLauncher,
                                   nsISupports *aContext,
                                   PRUint32 aForced)
{
    printf("ContentHandler::Show - Download\n");
    nsresult rv;

    mLauncher = aLauncher;

    //	mLauncher->SaveToDisk(nsnull, PR_FALSE);

    rv = GetDownloadInfo ();

    return NS_OK;
}

// get information related to donwload (mimetype, filename, server..)
NS_METHOD ContentHandler::GetDownloadInfo (void)
{
    printf("ContentHandler::GetDownloadInfo\n");
    nsresult rv;

    // create a GMozillaTransferItem object
    GObject* instance = G_OBJECT(GET_NEW);

    // get file mimetype
    nsCOMPtr<nsIMIMEInfo> MIMEInfo;
    rv = mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
    rv = MIMEInfo->GetMIMEType (mMimeType);
    G_MOZILLA_TRANSFER_ITEM(instance)->file_type = mMimeType.get();

    // get file name
    rv = mLauncher->GetSuggestedFileName (mFilename);
    mCFilename = NS_ConvertUTF16toUTF8(mFilename);
    G_MOZILLA_TRANSFER_ITEM(instance)->file_name = mCFilename.get();

    // get source url (concatened to file name)
    rv = mLauncher->GetSource(getter_AddRefs(mUri));
    rv = mUri->Resolve(NS_LITERAL_CSTRING("."), mSpec);
    G_MOZILLA_TRANSFER_ITEM(instance)->server = g_strconcat(mSpec.get(), mCFilename.get(), NULL);

    // TESTING TARGET FOLDER/FILE
    //nsCOMPtr<nsILocalFile> mFile(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
    //rv = mFile->InitWithNativePath(NS_LITERAL_CSTRING("/home/tnoleto/"));
    //rv = mFile->AppendRelativeNativePath(NS_LITERAL_CSTRING("myDownload"));
    //rv = mFile->Create(nsIFile::NORMAL_FILE_TYPE, 0777);
    //rv = mLauncher->GetTargetFile(getter_AddRefs(mDestFile));

    // set download initialization and launcher to control each download
    G_MOZILLA_TRANSFER_ITEM(instance)->started = FALSE;
    G_MOZILLA_TRANSFER_ITEM(instance)->launcher = mLauncher;

    gTransferItemTemp = G_MOZILLA_TRANSFER_ITEM(instance);

    // creates a progress listener to follow the download
    nsCOMPtr<nsIWebProgressListener2> listener =
        new ProgressListener(G_MOZILLA_TRANSFER_ITEM(instance));

    // connects the listener to the launcher which controls the download
    rv = mLauncher->SetWebProgressListener(listener);

    return NS_OK;
}

NS_IMETHODIMP ContentHandler::PromptForSaveToFile(
    nsIHelperAppLauncher *aLauncher,
    nsISupports *aWindowContext,
    const PRUnichar *aDefaultFile,
    const PRUnichar *aSuggestedFileExtension,
    nsILocalFile **_retval)
{
    //printf("ContentHandler::PromptForSaveToFile\n");

    return NS_OK;
}

NS_IMETHODIMP ProgressListener::OnStatusChange(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, nsresult aStatus,
        const PRUnichar *aMessage)
{
    //printf("ProgressListener::OnStatusChange\n");

    if (NS_SUCCEEDED (aStatus))
        return NS_OK;
}

//nsIWebProgressListener Functions
NS_IMETHODIMP ProgressListener::OnStateChange(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, PRUint32 aStateFlags,
        nsresult aStatus)
{
    //printf("ProgressListener::OnStateChange\n");
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP ProgressListener::OnProgressChange(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, PRInt32 aCurSelfProgress,
        PRInt32 aMaxSelfProgress, PRInt32 aCurTotalProgress,
        PRInt32 aMaxTotalProgress)
{
    //printf("ProgressListener::OnProgressChange\n");
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP ProgressListener::OnLocationChange(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, nsIURI *location)
{
    //printf("ProgressListener::OnLocationChange\n");
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP ProgressListener::OnSecurityChange(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, PRUint32 state)
{
    //printf("ProgressListener::OnSecurityChange\n");
    return NS_ERROR_NOT_IMPLEMENTED;
}

// nsIWebProgressListener 2
NS_IMETHODIMP ProgressListener::OnProgressChange64(nsIWebProgress *aWebProgress,
        nsIRequest *aRequest, PRInt64 aCurSelfProgress,
        PRInt64 aMaxSelfProgress, PRInt64 aCurTotalProgress,
        PRInt64 aMaxTotalProgress)
{
    //printf("ProgressListener::OnProgressChange64 - progress: %d\n", aCurSelfProgress);
    if (mTransferItem->started == FALSE)
    {
        mTransferItem->request = aRequest;
        mTransferItem->started = TRUE;

        GMozillaEngine *engine = g_mozilla_get_current_engine();
        mTransferItem->file_size = aMaxSelfProgress;

        // emit signal to open file handler dialog
        g_signal_emit_by_name(engine->engine, "download_request",
                              mTransferItem->server,
                              mTransferItem->file_name,
                              mTransferItem->file_type,
                              mTransferItem->file_size,
                              1);
    }

    // emit signal to get download progress and displays on download list dialog
    g_signal_emit_by_name(mTransferItem, G_WEB_TRANSFER_ITEM_SIGNAL_PROGRESS,
                          aCurSelfProgress, aMaxSelfProgress, 0);

    // when download finishes (bytes downloaded = total file size), emit completed signal
    if (aCurSelfProgress == aMaxSelfProgress)
        g_signal_emit_by_name(mTransferItem, G_WEB_TRANSFER_ITEM_SIGNAL_COMPLETED);

    return NS_OK;
}

