/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillaengine.c: Mozilla Web Engine Interface Implementation
  * @version $Rev: 2122 $
  * @date $LastChangedDate: 2005-11-03 12:22:51 -0400 (Thu, 03 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gmozillaengine.h"
#include "gmozillaenginemarshal.h"
#include "gtkmozembedmarshal.h"
#include "gmozillaweb.h"
#include "gtkmozilla.h"
#include "gmozillasupport.h"

extern ConfigData *config;

// it works for now, however, it must be found a way to put this into local variables, related 
// to a specific embed widget.(each tab will have an separated embed and progress bar)
int total_requests = 0, cur_requests = 0;

// class stuff
static void g_mozilla_engine_class_init (GMozillaEngineClass *klass);
static void g_mozilla_engine_init (GMozillaEngine *gengineembed);

static void g_mozilla_engine_base_init (gpointer g_class);
static gint g_mozilla_engine_signals[LAST_SIGNAL] = {0};
static GtkMozEmbedClass *parent_class = NULL;

// redirecting flow of some singal stuff
static void g_mozilla_engine_load_url (GMozillaEngine *gmozengine, const gchar  *url);
static void g_mozilla_engine_net_stop (GtkMozEmbed *embed);
static void g_mozilla_engine_net_start (GtkMozEmbed *embed);
static void g_mozilla_engine_title (GtkMozEmbed *embed);
static void g_mozilla_engine_location (GtkMozEmbed *embed);
static void g_mozilla_engine_progress (GtkMozEmbed *embed, gint curprogress, gint maxprogress);
static void g_mozilla_engine_net_state (GtkMozEmbed *embed, gint flags, guint status);
static void g_mozilla_engine_net_state_all (GtkMozEmbed *embed, const char *aURI, gint flags, guint status);
static void g_mozilla_engine_build_progress(GtkMozEmbed *embed, gint flags); 

static void
g_mozilla_engine_destroy (GMozillaEngine *self)
{}

static void
g_mozilla_engine_load_url (GMozillaEngine *self, const gchar *url)
{
    GMOZ_TRACE("ENGINE. Load URL: %s", url);
    gtk_mozilla_load_url(GTK_WIDGET(self->engine), url);
}

static GWebStatus
g_mozilla_engine_load_follow_link (GMozillaEngine *self, const gchar *url)
{
    //return (GWebStatus) gtk_mozilla_web_engine_activate_highlighted_link(GTK_MOZILLAWEBENGINE(self->engine), url);
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_load_url_with_css (GMozillaEngine *self, const gchar *url, const gchar *cssfile)
{
    //return (GWebStatus) gtk_mozilla_web_engine_load_with_css(GTK_MOZILLAWEBENGINE(self->engine), url, cssfile);
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_mozilla_engine_load_url_with_js (GMozillaEngine *self, gboolean  enable)
{
    //gtk_mozilla_web_engine_set_ecmascript_enabled(GTK_MOZILLAWEBENGINE(self->engine), enable);
    GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_load_home_page (GMozillaEngine *self)
{
    //gtk_mozilla_web_engine_load_homepage(GTK_MOZILLAWEBENGINE(self->engine));
    gtk_mozilla_load_url(GTK_WIDGET(self->engine), config->home);
    //GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_stop_load (GMozillaEngine *self)
{
    gtk_mozilla_stop_load(GTK_WIDGET(self->engine));
}

static void
g_mozilla_engine_reload (GMozillaEngine *self)
{
    gtk_mozilla_reload(GTK_WIDGET(self->engine));
}

static void
g_mozilla_engine_go_back (GMozillaEngine *self, guint nsteps)
{
    gtk_mozilla_go_back(GTK_WIDGET(self->engine));
}

static void
g_mozilla_engine_go_forward (GMozillaEngine *self, guint nsteps)
{
    gtk_mozilla_go_forward(GTK_WIDGET(self->engine));
}

static gboolean
g_mozilla_engine_can_go_back (GMozillaEngine *self)
{
    return gtk_moz_embed_can_go_back(GTK_MOZ_EMBED(self->engine));
}

static gboolean
g_mozilla_engine_can_go_forward (GMozillaEngine *self)
{
    return gtk_moz_embed_can_go_forward(GTK_MOZ_EMBED(self->engine));
}

static void
g_mozilla_engine_set_zoom_level (GMozillaEngine *self, gint level)
{
    gtk_mozilla_set_zoom_scale (GTK_WIDGET(self->engine), level);
}

static GWebStatus
g_mozilla_engine_set_zoom_level_around_point (GMozillaEngine *self, guint x, guint y, gint level)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_compare_frames_zoom_level (GMozillaEngine *self, gboolean *equal)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_get_image_properties (GMozillaEngine *self, const gchar *url, gchar **file_type, guint *file_size, guint *width, guint *height)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_get_frame_properties (GMozillaEngine *self, gchar **file_type, guint *file_size, GWebEngineEncoding *file_encoding, guint *security, gboolean *automatic_encoding)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_load_image (GMozillaEngine *self, const gchar* url)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static gint
g_mozilla_engine_get_back_window_history_list (GMozillaEngine *self,
        GObject **history)
{
    GMOZ_LOG_NOT_IMP();
    return 0;
}

static gint
g_mozilla_engine_get_forward_window_history_list (GMozillaEngine *self,
        GObject **history)
{
    GMOZ_LOG_NOT_IMP();
    return 0;
}

static guint
g_mozilla_engine_get_zoom_level (GMozillaEngine *self)
{
    return gtk_mozilla_get_zoom_scale (GTK_WIDGET(self->engine));
}

static void
g_mozilla_engine_zoom_by_steps (GMozillaEngine *self, guint relative_zoom, guint* legal_zoom_value)
{
    GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_set_frame_zoom_level (GMozillaEngine *self, guint level)
{
    GMOZ_LOG_NOT_IMP();
}

static guint
g_mozilla_engine_get_frame_zoom_level (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return 0;
}

static void
g_mozilla_engine_set_layout_mode (GMozillaEngine *self, GWebEngineLayoutMode mode)
{
    GMOZ_LOG_NOT_IMP();
}

static GWebEngineLayoutMode
g_mozilla_engine_get_layout_mode (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEBENGINE_LAYOUT_NORMAL;
}

static void
g_mozilla_engine_set_load_images (GMozillaEngine *self, GWebEngineImagePolicy level)
{
    GMOZ_LOG_NOT_IMP();
}

static GWebEngineImagePolicy
g_mozilla_engine_get_load_images (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEBENGINE_POLICY_ALL_IMAGES;
}

static void
g_mozilla_engine_set_frame_stacking (GMozillaEngine *self, gboolean enabled)
{
    GMOZ_LOG_NOT_IMP();
}

static gboolean
g_mozilla_engine_get_frame_stacking (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return TRUE;
}

static GWebStatus
g_mozilla_engine_save_web_page (GMozillaEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_save_current_frame (GMozillaEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static const gchar*
g_mozilla_engine_get_frame_title (GMozillaEngine *self, gint frame_number)
{
    return gtk_mozilla_get_frame_title(GTK_WIDGET(self->engine));
}

static gint
g_mozilla_engine_get_frame_number (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return 0;
}

static GWebStatus
g_mozilla_engine_save_web_page_image (GMozillaEngine *self, const gchar *url, const gchar* directory)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_mozilla_engine_select_all_text (GMozillaEngine *self)
{
    gtk_mozilla_select_all (GTK_WIDGET (self->engine));
}

static void
g_mozilla_engine_set_language_encoding (GMozillaEngine *self, GWebEngineEncoding encoding)
{
    switch (encoding)
    {
    case G_WEBENGINE_ENCODING_UTF8:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine),"UTF-8");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_UTF16:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "UTF-16BE");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_1:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-1");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_15:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-15");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_2:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-2");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_7:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-7");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_9:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-9");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1250:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1250");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1251:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1251");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1253:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1253");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1254:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1254");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_KOI8_R:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "KOI8-R");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    default:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine),
                                  gtk_mozilla_get_encoding (GTK_WIDGET(self->engine)));
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    }
}

static void
g_mozilla_engine_set_frame_encoding (GMozillaEngine *self, GWebEngineEncoding encoding)
{
    switch (encoding)
    {
    case G_WEBENGINE_ENCODING_UTF8:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine),"UTF-8");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_UTF16:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "UTF-16BE");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_1:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-1");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_15:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-15");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_2:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-2");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_7:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-7");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_ISO8859_9:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-9");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1250:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1250");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1251:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1251");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1253:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1253");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_CP_1254:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "windows-1254");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    case G_WEBENGINE_ENCODING_KOI8_R:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine), "KOI8-R");
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    default:
        gtk_mozilla_set_encoding (GTK_WIDGET(self->engine),
                                  gtk_mozilla_get_encoding (GTK_WIDGET(self->engine)));
        gtk_mozilla_reload (GTK_WIDGET(self->engine));
        break;
    }
}

static gboolean
g_mozilla_engine_find_text (GMozillaEngine *self, const gchar *string, gboolean reverse, gboolean whole_word, gboolean case_sensitive, gboolean restart)
{
    return gtk_mozilla_find_text (GTK_WIDGET (self->engine),
                           string, reverse, whole_word,
                           case_sensitive, restart);
}

static void
g_mozilla_engine_authenticate (GMozillaEngine *self, gboolean ok, const gchar *username, const gchar *password)
{
    GMOZ_LOG_NOT_IMP();
}

static GWebStatus
g_mozilla_engine_cut (GMozillaEngine *self)
{
    gtk_mozilla_cut (GTK_WIDGET (self->engine));
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_copy (GMozillaEngine *self)
{
    gtk_mozilla_copy (GTK_WIDGET (self->engine));
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_paste (GMozillaEngine *self)
{
    gtk_mozilla_paste (GTK_WIDGET (self->engine));
    return G_WEB_STATUS_OK;
}

static gboolean
g_mozilla_engine_can_cut (GMozillaEngine *self)
{
    return gtk_mozilla_can_cut (GTK_WIDGET (self->engine));
}

static gboolean
g_mozilla_engine_can_paste (GMozillaEngine *self)
{
    return gtk_mozilla_can_paste (GTK_WIDGET (self->engine));
}

static gboolean
g_mozilla_engine_can_copy (GMozillaEngine *self)
{
    return gtk_mozilla_can_copy (GTK_WIDGET (self->engine));
}

static gboolean
g_mozilla_engine_can_selecttext (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
    return TRUE;
}

static gchar*
g_mozilla_engine_get_favicon_name (GMozillaEngine *self, const gchar *favicon_folder)
{/*
            GdkPixbuf* icon;
            GError *error = NULL;
            gchar *file_path=NULL;
            //gtk_mozilla_web_engine_get_icon(GTK_MOZILLAWEBENGINE(self->engine), &icon);
            if ( icon != NULL ) {
                time_t timer;
                timer = time(NULL);
                file_path = g_strdup_printf("%s/icon_%d.png", favicon_folder, timer);
                if (gdk_pixbuf_get_width (icon) > 16 || gdk_pixbuf_get_height (icon) > 16) {
                       GdkPixbuf *scaled_icon = gdk_pixbuf_scale_simple (icon, 16, 16, GDK_INTERP_BILINEAR);
                       icon = scaled_icon;
                }
                gdk_pixbuf_save(icon, file_path, "png", &error, NULL);
                g_object_unref(icon);
            }
            return file_path;*/
    GMOZ_LOG_NOT_IMP();
    return NULL;
}

static GObject*
g_mozilla_engine_get_engine_widget (GMozillaEngine *self)
{
    char *home_path;
    char *full_path;

    create_config_directory ();
    open_and_load_config_file ();
    mozilla_register_components ();
    cur_engine=self;
    write_config ();

    home_path = PR_GetEnv("HOME");
    if (!home_path) {
      fprintf(stderr, "Failed to get HOME\n");
      exit(1);
    }

    full_path = g_strdup_printf("%s/%s", home_path, ".mozilla");
    gtk_mozilla_set_profile (full_path, "maemo-browser-mozilla");

    // activating snav - IT'S NOT WORKING - MOZILLA/MINIMO BUG
    //gtk_mozilla_preference_set_int ("snav.keyCode.modifier",0);

    /*gtk_mozilla_preference_set_boolean ("browser.display.use_focus_colors", TRUE);
    gtk_mozilla_preference_set ("browser.display.focus_text_color", "#9d0202");
    gtk_mozilla_preference_set ("browser.display.focus_background_color", "#f6ff08");
    gtk_mozilla_preference_set_int ("browser.display.focus_ring_width", 3);*/
gtk_mozilla_preference_set_int ("network.proxy.type", 1);
gtk_mozilla_preference_set ("network.proxy.http", "172.18.110.24");
gtk_mozilla_preference_set_int ("network.proxy.http_port", 8080);

    // accept redirects
    gtk_mozilla_preference_set_boolean ("browser.accept.redirects", TRUE);

    //Content refreshes
    gtk_mozilla_preference_set_boolean ("browser.accept.refreshes", TRUE);

    return self->engine;
}

static void
g_mozilla_engine_set_open_link_mode (GMozillaEngine *self, gboolean in_new_window)
{
    GMOZ_LOG_NOT_IMP();
}

static GWebStatus
g_mozilla_engine_set_true_zoom_mode (GMozillaEngine *self, gboolean status)
{
    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_mozilla_engine_notify_plugins (GMozillaEngine *self, gint event)
{
    GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_preload_current_skin (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_reset (GMozillaEngine *self)
{
    GMOZ_LOG_NOT_IMP();
}

static void
g_mozilla_engine_interface_init (GWebEngineIface *iface)
{
    iface->destroy                         = (void (*) (GWebEngine*))
            g_mozilla_engine_destroy;
    iface->reset                           = (void (*) (GWebEngine*))
            g_mozilla_engine_reset;
    iface->load_url                        = (void (*) (GWebEngine*, const gchar*))
            g_mozilla_engine_load_url;
    iface->load_follow_link                = (GWebStatus (*) (GWebEngine*, const gchar*))
            g_mozilla_engine_load_follow_link;
    iface->load_url_with_css               = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
            g_mozilla_engine_load_url_with_css;
    iface->load_url_with_js                = (void (*) (GWebEngine*, gboolean))
            g_mozilla_engine_load_url_with_js;
    iface->load_home_page                  = (void (*) (GWebEngine*))
            g_mozilla_engine_load_home_page;
    iface->stop_load                       = (void (*) (GWebEngine*))
            g_mozilla_engine_stop_load;
    iface->reload                          = (void (*) (GWebEngine*))
            g_mozilla_engine_reload;
    iface->go_back                         = (void (*) (GWebEngine*, guint))
            g_mozilla_engine_go_back;
    iface->go_forward                      = (void (*) (GWebEngine*, guint))
            g_mozilla_engine_go_forward;
    iface->can_go_back                     = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_go_back;
    iface->can_go_forward                  = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_go_forward;
    iface->set_zoom_level                  = (void (*) (GWebEngine*, gint ))
            g_mozilla_engine_set_zoom_level;
    iface->zoom_by_steps                    = (void (*) (GWebEngine*, guint, guint *))
            g_mozilla_engine_zoom_by_steps;
    iface->set_zoom_level_around_point     = (GWebStatus (*) (GWebEngine*, guint, guint, gint))
            g_mozilla_engine_set_zoom_level_around_point;
    iface->compare_frames_zoom_level       = (GWebStatus (*) (GWebEngine*, gboolean*))
            g_mozilla_engine_compare_frames_zoom_level;
    iface->get_image_properties            = (GWebStatus (*) (GWebEngine*, const gchar*, gchar**, guint*, guint*, guint*))
            g_mozilla_engine_get_image_properties;
    iface->get_frame_properties            = (GWebStatus (*) (GWebEngine*, gchar**, guint*, GWebEngineEncoding*, guint*, gboolean*))
            g_mozilla_engine_get_frame_properties;
    iface->load_image                      = (GWebStatus (*) (GWebEngine*, const gchar*))
            g_mozilla_engine_load_image;
    iface->get_back_window_history_list    = (gint (*) (GWebEngine*, GObject**))
            g_mozilla_engine_get_back_window_history_list;
    iface->get_forward_window_history_list = (gint (*) (GWebEngine*, GObject**))
            g_mozilla_engine_get_forward_window_history_list;
    iface->get_zoom_level                  = (guint (*) (GWebEngine*))
            g_mozilla_engine_get_zoom_level;
    iface->set_frame_zoom_level            = (void (*) (GWebEngine*, guint))
            g_mozilla_engine_set_frame_zoom_level;
    iface->get_frame_zoom_level            = (guint (*) (GWebEngine*))
            g_mozilla_engine_get_frame_zoom_level;
    iface->set_layout_mode                 = (void (*) (GWebEngine*, GWebEngineLayoutMode))
            g_mozilla_engine_set_layout_mode;
    iface->get_layout_mode                 = (GWebEngineLayoutMode (*) (GWebEngine*))
            g_mozilla_engine_get_layout_mode;
    iface->set_load_images                 = (void (*) (GWebEngine*, GWebEngineImagePolicy))
            g_mozilla_engine_set_load_images;
    iface->get_load_images                 = (GWebEngineImagePolicy (*) (GWebEngine *self))
            g_mozilla_engine_get_load_images;
    iface->set_frame_stacking              = (void (*) (GWebEngine*, gboolean))
            g_mozilla_engine_set_frame_stacking;
    iface->get_frame_stacking              = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_get_frame_stacking;
    iface->save_web_page                   = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
            g_mozilla_engine_save_web_page;
    iface->save_current_frame              = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
            g_mozilla_engine_save_current_frame;
    iface->get_frame_title                 = (const gchar* (*) (GWebEngine*, gint))
            g_mozilla_engine_get_frame_title;
    iface->get_frame_number                = (gint (*) (GWebEngine*))
            g_mozilla_engine_get_frame_number;
    iface->save_web_page_image             = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
            g_mozilla_engine_save_web_page_image;
    iface->select_all_text                 = (void (*) (GWebEngine*))
            g_mozilla_engine_select_all_text;
    iface->set_language_encoding           = (void (*) (GWebEngine*, GWebEngineEncoding))
            g_mozilla_engine_set_language_encoding;
    iface->set_frame_encoding              = (void (*) (GWebEngine*, GWebEngineEncoding))
            g_mozilla_engine_set_frame_encoding;
    iface->find_text                       = (gboolean (*) (GWebEngine*, const gchar*, gboolean, gboolean, gboolean, gboolean))
            g_mozilla_engine_find_text;
    iface->authenticate                    = (void (*) (GWebEngine*, gboolean, const gchar*, const gchar*))
            g_mozilla_engine_authenticate;
    iface->cut                             = (GWebStatus (*) (GWebEngine*))
            g_mozilla_engine_cut;
    iface->copy                            = (GWebStatus (*) (GWebEngine*))
            g_mozilla_engine_copy;
    iface->paste                           = (GWebStatus (*) (GWebEngine*))
            g_mozilla_engine_paste;
    iface->can_cut                         = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_cut;
    iface->can_paste                       = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_paste;
    iface->can_copy                        = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_copy;
    iface->can_selecttext                  = (gboolean (*) (GWebEngine*))
            g_mozilla_engine_can_selecttext;
    iface->get_favicon_name                = (gchar* (*) (GWebEngine*, const gchar*))
            g_mozilla_engine_get_favicon_name;
    iface->get_engine_widget               = (GObject* (*) (GWebEngine*))
            g_mozilla_engine_get_engine_widget;
    iface->set_open_link_mode              = (void (*) (GWebEngine*, gboolean))
            g_mozilla_engine_set_open_link_mode;
    iface->set_true_zoom_mode              = (GWebStatus (*) (GWebEngine*, gboolean))
            g_mozilla_engine_set_true_zoom_mode;
    iface->notify_plugins                  = (void (*) (GWebEngine*, gint))
            g_mozilla_engine_notify_plugins;
    iface->preload_current_skin            = (void (*) (GWebEngine*))
            g_mozilla_engine_preload_current_skin;
}

static void
g_mozilla_engine_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    G_MOZILLA_ENGINE(instance)->engine = NULL;
}

GType
g_mozilla_engine_get_type (void)
{
    static GType type = 0;
    if (!type)
    {
        static GTypeInfo const object_info =
            {
                sizeof (GMozillaEngineClass),

                (GBaseInitFunc) g_mozilla_engine_base_init,
                (GBaseFinalizeFunc) NULL,

                (GClassInitFunc) g_mozilla_engine_class_init,
                (GClassFinalizeFunc) NULL,
                NULL,

                sizeof (GMozillaEngine),
                0,
                (GInstanceInitFunc) g_mozilla_engine_instance_init,
            };

        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_mozilla_engine_interface_init,
                NULL,
                NULL
            };

        type = g_type_register_static (GTK_TYPE_MOZ_EMBED, "GMozillaEngine", &object_info, (GTypeFlags)0);

        g_type_add_interface_static (type,
                                     G_TYPE_WEB_ENGINE,
                                     &iface_info);
    }
    return type;
}

static void
g_mozilla_engine_base_init (gpointer g_class)
{
    static gboolean initialized = FALSE;

    if (!initialized)
    {
        /* create interface signals here. */
       g_mozilla_engine_signals[URL_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_URL_CHANGED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GMozillaEngineClass, location),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_mozilla_engine_signals[TITLE_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_TITLE,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GMozillaEngineClass, title),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_mozilla_engine_signals[LOADING_PROGRESS]
        = g_signal_new (G_WEBWIDGET_SIGNAL_LOADING_PROGRESS,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GMozillaEngineClass, progress),
                        NULL, NULL,
                        gtk_marshal_VOID__INT_INT,
                        G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);

        g_mozilla_engine_signals[START_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_START_LOADING,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GMozillaEngineClass, net_start),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_mozilla_engine_signals[FINISHED_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FINISHED_LOADING,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GMozillaEngineClass, net_stop),
                        NULL, NULL,
                        gtk_marshal_VOID__INT_INT_BOOL,
                        G_TYPE_NONE,
                        3,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_BOOLEAN);

  g_mozilla_engine_signals[WINDOW_REQUEST] =
    g_signal_new(G_WEBWIDGET_SIGNAL_WINDOW_REQUEST,
		   G_TYPE_MOZILLA_ENGINE,
		   G_SIGNAL_RUN_FIRST,
		   G_STRUCT_OFFSET(GMozillaEngineClass, window_request),
                   NULL, NULL,
		   gtk_marshal_VOID__POINTER_STRING_BOOL,
		   GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_STRING, GTK_TYPE_BOOL);


  g_mozilla_engine_signals[DOWNLOAD_REQUEST] =
    g_signal_new(G_WEBWIDGET_SIGNAL_DOWNLOAD_REQUEST,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, download_request),
                   NULL, NULL,
		   gtk_marshal_VOID__STRING_STRING_STRING_INT_INT,
		   GTK_TYPE_NONE, 5, GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE, 
				GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
				GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
				GTK_TYPE_INT, GTK_TYPE_INT);

  g_mozilla_engine_signals[ENCODING] =
    g_signal_new(G_WEBWIDGET_SIGNAL_ENCODING,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, encoding),
                   NULL, NULL,
		   gtk_marshal_VOID__UINT,
		   GTK_TYPE_NONE, 1, GTK_TYPE_UINT);

  g_mozilla_engine_signals[AUTHENTIFICATION_REQUIRED] =
    g_signal_new(G_WEBWIDGET_SIGNAL_AUTHENTIFICATION_REQUIRED,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, authentification_required),
                   NULL, NULL,
		   gtk_marshal_VOID__INT_STRING_STRING,
                   G_TYPE_NONE, 3,
                   G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);

  g_mozilla_engine_signals[CONFIRM] =
    g_signal_new(G_WEBWIDGET_SIGNAL_CONFIRM,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, confirm),
                   NULL, NULL,
		   gtk_marshal_BOOLEAN__STRING,
                   G_TYPE_BOOLEAN, 1, G_TYPE_STRING);

  g_mozilla_engine_signals[PROMPT] =
    g_signal_new(G_WEBWIDGET_SIGNAL_PROMPT,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, prompt),
                   NULL, NULL,
		   gtk_marshal_STRING__STRING_STRING,
                   G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);

  g_mozilla_engine_signals[UPLOAD_DIALOG] =
    g_signal_new(G_WEBWIDGET_SIGNAL_UPLOAD_DIALOG,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, upload_dialog),
                   NULL, NULL,
		   gtk_marshal_BOOL__STRING_STRING_POINTER,
                   G_TYPE_BOOLEAN, 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);

  g_mozilla_engine_signals[FILE_UPLOAD_PROGRESS] =
    g_signal_new(G_WEBWIDGET_SIGNAL_FILE_UPLOAD_PROGRESS,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, file_upload_progress),
                   NULL, NULL,
		   gtk_marshal_VOID__INT_STRING,
                   G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_STRING);

  g_mozilla_engine_signals[WEBENGINE_POPUP_MENU] =
    g_signal_new(G_WEBWIDGET_SIGNAL_WEBENGINE_POPUP_MENU,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, webengine_popup_menu),
                   NULL, NULL,
		   gtk_marshal_VOID__UINT_INT_INT_STRING_STRING_STRING_STRING,
                   G_TYPE_NONE, 7, G_TYPE_UINT, G_TYPE_INT, G_TYPE_INT, 
                   G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  g_mozilla_engine_signals[MODAL_DIALOG] =
    g_signal_new(G_WEBWIDGET_SIGNAL_MODAL_DIALOG,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, modal_dialog),
                   NULL, NULL,
		   gtk_marshal_INT__STRING_STRING_INT_INT_INT_INT,
                   G_TYPE_INT,
                   6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
                   G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);

  g_mozilla_engine_signals[MODAL_CERTIFICATE_DIALOG] =
    g_signal_new(G_WEBWIDGET_SIGNAL_MODAL_CERTIFICATE_DIALOG,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, modal_certificate_dialog),
                   NULL, NULL,
		   gtk_marshal_INT__STRING_INT_INT_INT_INT_INT,
                   G_TYPE_INT, 
		   6, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);

  g_mozilla_engine_signals[ALERT] =
    g_signal_new(G_WEBWIDGET_SIGNAL_ALERT,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, alert),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__STRING,
                   G_TYPE_NONE, 1, G_TYPE_STRING);

  g_mozilla_engine_signals[MAILTO] =
    g_signal_new(G_WEBWIDGET_SIGNAL_MAILTO,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, mailto),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__STRING,
                   G_TYPE_NONE, 1, G_TYPE_STRING);

  g_mozilla_engine_signals[UNKNOWN_PROTOCOL] =
    g_signal_new(G_WEBWIDGET_SIGNAL_UNKNOWN_PROTOCOL,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, unknown_protocol),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__STRING,
                   G_TYPE_NONE, 1, G_TYPE_STRING);

  g_mozilla_engine_signals[RAISE_REQUEST] =
    g_signal_new(G_WEBWIDGET_SIGNAL_RAISE_REQUEST,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, raise_request),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);

  g_mozilla_engine_signals[LOWER_REQUEST] =
    g_signal_new(G_WEBWIDGET_SIGNAL_LOWER_REQUEST,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, lower_request),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);

  g_mozilla_engine_signals[FILE_UPLOAD_STARTED] =
    g_signal_new(G_WEBWIDGET_SIGNAL_FILE_UPLOAD_STARTED,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, file_upload_started),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

  g_mozilla_engine_signals[FILE_UPLOAD_FINISHED] =
    g_signal_new(G_WEBWIDGET_SIGNAL_FILE_UPLOAD_FINISHED,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, file_upload_finished),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

  g_mozilla_engine_signals[EXTERNAL_ACTION] =
    g_signal_new(G_WEBWIDGET_SIGNAL_EXTERNAL_ACTION,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, external_action),
                   NULL, NULL,
		   g_cclosure_marshal_VOID__STRING, 
                   G_TYPE_NONE, 1, G_TYPE_STRING);

  g_mozilla_engine_signals[FRAME_STACKING_CHANGED] =
    g_signal_new(G_WEBWIDGET_SIGNAL_FRAME_STACKING_CHANGED,
                   G_TYPE_MOZILLA_ENGINE,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, frame_stacking_changed),
                   NULL, NULL,
                   g_cclosure_marshal_VOID__BOOLEAN, 
		   G_TYPE_NONE, 1, G_TYPE_BOOLEAN);


  g_mozilla_engine_signals[SECURITY_MODE_CHANGED] =
      g_signal_new(G_WEBWIDGET_SIGNAL_SECURITY_MODE_CHANGED,
		   G_TYPE_MOZILLA_ENGINE,
		   G_SIGNAL_RUN_LAST,
		   G_STRUCT_OFFSET (GMozillaEngineClass, security_mode_changed),
                   NULL, NULL,
		   gtk_marshal_VOID__UINT,
		   GTK_TYPE_NONE, 1, GTK_TYPE_UINT);

  g_mozilla_engine_signals[CLOSE_WINDOW] =
    g_signal_new(G_WEBWIDGET_SIGNAL_CLOSE_WINDOW,
		   G_TYPE_MOZILLA_ENGINE,
		   G_SIGNAL_RUN_FIRST,
		   G_STRUCT_OFFSET(GMozillaEngineClass, close_window),
                   NULL, NULL,
		   gtk_marshal_NONE__NONE,
		   GTK_TYPE_NONE, 0);

    initialized = TRUE;
    }
}

static void
g_mozilla_engine_class_init (GMozillaEngineClass *klass)
{
    GObjectClass *gobject_class;
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;
    GtkMozEmbedClass *moz_embed_class;

    parent_class    = (GtkMozEmbedClass *) g_type_class_peek_parent (klass);

    gobject_class   = (GObjectClass *) klass;
    object_class    = (GtkObjectClass *) klass;
    widget_class    = (GtkWidgetClass *) klass;
    moz_embed_class = (GtkMozEmbedClass *) klass;

    // web spefic singals
    g_mozilla_engine_signals[ASK_COOKIE] =
        g_signal_new("ask-cookie",
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, ask_cookie),
                   NULL, NULL,
                   gtk_marshal_VOID__POINTER_INT_STRING_STRING_STRING_STRING_STRING_BOOLEAN_INT,
		   G_TYPE_NONE, 9,
		   GTK_TYPE_POINTER,
		   GTK_TYPE_INT,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_BOOL, 
		   GTK_TYPE_INT);

    g_mozilla_engine_signals[CERTIFICATE_DIALOG] =
        g_signal_new("certificate-dialog",
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, certificate_dialog),
                   NULL, NULL,
                   gtk_marshal_VOID__POINTER,
		   G_TYPE_NONE, 1, GTK_TYPE_POINTER);

    g_mozilla_engine_signals[CERTIFICATE_PASSWORD_DIALOG] =
        g_signal_new("certificate-password-dialog",
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, certificate_password_dialog),
                   NULL, NULL,
                   gtk_marshal_VOID__STRING_STRING_POINTER,
		   G_TYPE_NONE, 3, 
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_POINTER);

    g_mozilla_engine_signals[CERTIFICATE_DETAILS] =
        g_signal_new("certificate-details",
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GMozillaEngineClass, certificate_details),
                   NULL, NULL,
                   gtk_marshal_VOID__POINTER,
		   G_TYPE_NONE, 1, GTK_TYPE_POINTER);

    moz_embed_class->net_stop = g_mozilla_engine_net_stop;
    moz_embed_class->net_start = g_mozilla_engine_net_start;
    moz_embed_class->title = g_mozilla_engine_title;
    moz_embed_class->location = g_mozilla_engine_location;
    moz_embed_class->progress = g_mozilla_engine_progress;
    moz_embed_class->net_state = g_mozilla_engine_net_state;
    moz_embed_class->net_state_all = g_mozilla_engine_net_state_all;
}

/* a macro for creating a new object of our type */
#define GET_MOZILLA_ENGINE_NEW (g_object_new(g_mozilla_engine_get_type(), NULL))

static void
g_mozilla_engine_init (GMozillaEngine *gengineembed)
{
    // FIX ME - the line bellow fires an 'assert' on the console during the browser startup
    //GLIB CRITICAL ** default - void gtk_moz_embed_load_url(GtkMozEmbed*, const char*): assertion `embed != NULL' failed

    //g_mozilla_engine_load_url (G_MOZILLA_ENGINE(gengineembed), "about:blank");
}

static void
g_mozilla_engine_net_start (GtkMozEmbed *embed)
{
    cur_requests = 0;
    total_requests = 0;
//printf ("[MOZILLA EAL] g_mozilla_engine_net_start \n");
    gchar *submitted_uri = g_strdup (gtk_moz_embed_get_submitted_uri (embed)); 
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_START_LOADING, submitted_uri);
    g_free (submitted_uri);
}

static void
g_mozilla_engine_progress (GtkMozEmbed *embed, gint curprogress, gint maxprogress)
{
    // the signal is now emitted from g_mozilla_engine_build_progress inside g_mozilla_engine_net_state_all  

    //printf ("[MOZILLA EAL]g_mozilla_engine_progress '%d' '%d'\n", curprogress, maxprogress);
    // g_signal_emit_by_name(GTK_OBJECT(embed), G_WEBWIDGET_SIGNAL_LOADING_PROGRESS, curprogress, maxprogress);
}

static void
g_mozilla_engine_net_stop (GtkMozEmbed *embed)
{
    //printf ("[MOZILLA EAL]g_mozilla_engine_net_stop\n");
    // TESTING - gint finish_status, gint page_size, gboolean has_frames 
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_FINISHED_LOADING, 0, 15000, TRUE);

    total_requests = 0;
    cur_requests = 0;
}

static void
g_mozilla_engine_location (GtkMozEmbed *embed)
{
    //printf ("[MOZILLA EAL]g_mozilla_engine_location'\n");
    gchar * location = g_strdup (gtk_moz_embed_get_location(embed));
    g_signal_emit_by_name(GTK_OBJECT(embed), G_WEBWIDGET_SIGNAL_URL_CHANGED, location);
    g_free(location);
}

static void
g_mozilla_engine_title (GtkMozEmbed *embed)
{
    //printf ("[MOZILLA EAL]g_mozilla_engine_title\n");
    gchar * title = g_strdup (gtk_moz_embed_get_title(embed));
    g_signal_emit_by_name(GTK_OBJECT(embed), G_WEBWIDGET_SIGNAL_TITLE, title);
    g_free(title);
}

static void g_mozilla_engine_net_state (GtkMozEmbed *embed, gint flags, guint status)
{
    //printf ("[MOZILLA EAL] NET_STATE \n");
}

static void g_mozilla_engine_net_state_all (GtkMozEmbed *embed, const char *aURI, gint flags, guint status)
{
    //printf ("[MOZILLA EAL] NET_STATE_ALL!!! \n");
    g_mozilla_engine_build_progress(embed, flags);
}

static void g_mozilla_engine_build_progress(GtkMozEmbed *embed, gint flags) 
{
    
    if (flags & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
    {
        if (flags & GTK_MOZ_EMBED_FLAG_START)
        {
            total_requests ++;
        }
        else if (flags & GTK_MOZ_EMBED_FLAG_STOP)
        {
            cur_requests++;
        }
    }
    //printf("cur_requests: %d - total_requests: %d\n", cur_requests, total_requests);
  
    g_signal_emit_by_name(GTK_OBJECT(embed), G_WEBWIDGET_SIGNAL_LOADING_PROGRESS, cur_requests, total_requests);
}

GObject*
g_mozilla_engine_new (void)
{
    GMozillaEngine *instance = G_MOZILLA_ENGINE(GET_MOZILLA_ENGINE_NEW);
    G_MOZILLA_ENGINE(instance)->engine = G_OBJECT(instance);
    
    g_signal_connect(G_MOZILLA_ENGINE(instance)->engine, "dom_mouse_down", G_CALLBACK(on_button_pressed_cb), G_MOZILLA_ENGINE(instance));
    g_signal_connect(G_MOZILLA_ENGINE(instance)->engine, "dom_mouse_up", G_CALLBACK(on_button_released_cb), G_MOZILLA_ENGINE(instance));
  
    return G_OBJECT(instance);
}

GObject*
g_mozilla_engine_new_with_context (gpointer context)
{
    /* FIX ME */    
    return g_mozilla_engine_new();
}

GObject*
g_mozilla_engine_new_with_context_and_tracks (gpointer context,
        gboolean leave_tracks)
{
    /* FIX ME */
    return g_mozilla_engine_new();
}

GMozillaEngine*
g_mozilla_get_current_engine(void)
{
    return cur_engine;
}

void
clear_ctx_data(GMozillaEngine *gengineembed)
{
    GContextData *data = &gengineembed->ctx_data;
    if (data->href)
        g_free(data->href);
    if (data->framehref)
        g_free(data->framehref);
    if (data->imghref)
        g_free(data->imghref);
    if (data->linktext)
        g_free(data->linktext);
    data->href = NULL;
    data->imghref = NULL;
    data->framehref = NULL;
    data->linktext = NULL;
    data->xcoord = 0;
    data->ycoord = 0;
    data->ctx_type = 0;
}

gboolean timer_out (gpointer data)
{
    GMozillaEngine *gengineembed = (GMozillaEngine *)data;

    g_signal_emit(gengineembed, g_mozilla_engine_signals[WEBENGINE_POPUP_MENU], 0,
                  gengineembed->ctx_data.ctx_type,
                  gengineembed->ctx_data.xcoord,
                  gengineembed->ctx_data.ycoord,
                  gengineembed->ctx_data.linktext,
                  gengineembed->ctx_data.href,
                  gengineembed->ctx_data.imghref,
                  gengineembed->ctx_data.framehref);
 
 
    nsIWebBrowser *webBrowser = nsnull;
    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(gengineembed), &webBrowser);
 
    nsCOMPtr<nsIDOMWindow> DOMWindow;
    webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
 
    nsCOMPtr<nsISelection> selection;
    nsresult rv = DOMWindow->GetSelection(getter_AddRefs(selection));
    if (NS_SUCCEEDED(rv))
    {
        selection->RemoveAllRanges();
    }

    return FALSE;
}
 
gint on_button_pressed_cb(GtkWidget *button, gpointer *dom_event, GMozillaEngine *gengineembed)
{
    clear_ctx_data(gengineembed);
 
    ((nsIDOMMouseEvent*)dom_event)->GetClientX(&gengineembed->ctx_data.xcoord);
    ((nsIDOMMouseEvent*)dom_event)->GetClientY(&gengineembed->ctx_data.ycoord);
 
    gengineembed->ctx_data.ctx_type =
         gtk_mozilla_get_context_menu_type(GTK_WIDGET(gengineembed), dom_event, &gengineembed->ctx_data);
 
    if (gengineembed->ctx_data.ctx_type & G_WEBENGINE_POPUPMENU_DOCUMENT)
    {
        gengineembed->timer_id = g_timeout_add ( 1000, timer_out , gengineembed);
    }

    return FALSE; 
}
 
gint on_button_released_cb(GtkWidget *button, gpointer *dom_event, GMozillaEngine *gengineembed)
{
    if (gengineembed->timer_id)
    {
        g_source_remove (gengineembed->timer_id);
        gengineembed->timer_id = 0;

        return TRUE;
    }

    return FALSE;
}
