/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillahistorylist.c: Mozilla History List Interface Implementation
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 09:35:49 -0400 (Wed, 02 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include "gmozillahistorylist.h"

static void
g_mozilla_history_list_destroy (GMozillaHistoryList *self)
{
    gtk_mozilla_web_delete_history_list(G_OBJECT(self));
}

static void
g_mozilla_history_list_delete_item (GMozillaHistoryList *self, GObject *item)
{
    gtk_mozilla_web_remove_from_global_history(G_OBJECT(self), (GtkWebHistoryItem*)item);
}

static GObject*
g_mozilla_history_list_get_items (GMozillaHistoryList *self)
{
    return G_MOZILLA_HISTORY_LIST(self)->items;
}

static gint
g_mozilla_history_list_get_items_count (GMozillaHistoryList *self)
{
    return G_MOZILLA_HISTORY_LIST(self)->num_items;
}

static gboolean
g_mozilla_history_list_position_ok (GMozillaHistoryList *self, gint pos)
{
    return (G_MOZILLA_HISTORY_LIST(self)->items &&
            pos >= 0 &&
            pos < G_MOZILLA_HISTORY_LIST(self)->num_items);
}

static const gchar*
g_mozilla_history_list_get_item_title (GMozillaHistoryList *self, gint pos)
{
    if (g_mozilla_history_list_position_ok(self, pos))
    {
        return ((GtkWebHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].title;
    }
    return NULL;
}

static const gchar*
g_mozilla_history_list_get_item_url (GMozillaHistoryList *self, gint pos)
{
    if (g_mozilla_history_list_position_ok(self, pos))
    {
        return ((GtkWebHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].url;
    }
    return NULL;
}

static GTime
g_mozilla_history_list_get_item_time (GMozillaHistoryList *self, gint pos)
{
    if (g_mozilla_history_list_position_ok(self, pos))
    {
        return ((GtkWebHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].accessed;
    }
    return 0;
}

static void
g_mozilla_history_list_interface_init (GWebHistoryListIface *iface)
{
    iface->destroy         = (void (*) (GWebHistoryList*))
                             g_mozilla_history_list_destroy;
    iface->delete_item     = (void (*) (GWebHistoryList*, GObject*))
                             g_mozilla_history_list_delete_item;
    iface->get_items       = (GObject* (*) (GWebHistoryList*))
                             g_mozilla_history_list_get_items;
    iface->get_items_count = (gint (*) (GWebHistoryList*))
                             g_mozilla_history_list_get_items_count;
    iface->get_item_title  = (const gchar* (*) (GWebHistoryList*, gint))
                             g_mozilla_history_list_get_item_title;
    iface->get_item_url    = (const gchar* (*) (GWebHistoryList*, gint))
                             g_mozilla_history_list_get_item_url;
    iface->get_item_time   = (GTime (*) (GWebHistoryList*, gint))
                             g_mozilla_history_list_get_item_time;
}

static void
g_mozilla_history_list_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    G_MOZILLA_HISTORY_LIST(instance)->items     = NULL;
    G_MOZILLA_HISTORY_LIST(instance)->num_items = 0;
}

GType
g_mozilla_history_list_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
        {
            sizeof (GMozillaHistoryListClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            NULL,   /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof (GMozillaHistoryList),
            0,      /* n_preallocs */
            (GInstanceInitFunc) g_mozilla_history_list_instance_init    /* instance_init */
        };
        static const GInterfaceInfo iface_info =
        {
            (GInterfaceInitFunc) g_mozilla_history_list_interface_init,
            NULL,   /* interface_finalize */
            NULL    /* interface_data */
        };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GMozillaHistoryList",&info,
                                       (GTypeFlags) 0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_HISTORY_LIST,
                                     &iface_info);
    }
    return type;
}

GObject*
g_mozilla_history_list_new (void)
{

    GObject* instance = G_OBJECT(g_object_new(G_TYPE_MOZILLA_HISTORY_LIST,NULL));
    G_MOZILLA_HISTORY_LIST(instance)->items = (GObject*) g_try_malloc (sizeof (GtkWebHistoryItem)*1000);
    gtk_mozilla_web_get_history_list((GtkWebHistoryItem*)G_MOZILLA_HISTORY_LIST(instance)->items, G_MOZILLA_HISTORY_LIST(instance)->num_items);
    
    return instance;
}
