/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillapluginlist.c: Mozilla Plugin List Interface Implementation
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 15:35:49 +0200 (Wed, 02 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include "gmozillapluginlist.h"

/* GMozillaPluginList */

static void
g_mozilla_plugin_list_destroy (GMozillaPluginList *self)
{}

static GObject*
g_mozilla_plugin_list_get_items (GMozillaPluginList *self)
{
    return G_MOZILLA_PLUGIN_LIST(self)->items;
}

static gint
g_mozilla_plugin_list_get_items_count (GMozillaPluginList *self)
{
    return G_MOZILLA_PLUGIN_LIST(self)->num_items;
}

static gboolean
g_mozilla_plugin_list_position_ok (GMozillaPluginList *self, gint pos)
{
    return (G_MOZILLA_PLUGIN_LIST(self)->items &&
            pos >= 0 &&
            pos < G_MOZILLA_PLUGIN_LIST(self)->num_items);
}

static const gchar*
g_mozilla_plugin_list_get_item_title (GMozillaPluginList *self, gint pos)
{}

static const gchar*
g_mozilla_plugin_list_get_item_type (GMozillaPluginList *self, gint pos)
{}

static void
g_mozilla_plugin_list_interface_init (GWebPluginListIface *iface)
{
    iface->destroy         = (void (*) (GWebPluginList*))
                             g_mozilla_plugin_list_destroy;
    iface->get_items       = (GObject* (*) (GWebPluginList*))
                             g_mozilla_plugin_list_get_items;
    iface->get_items_count = (gint (*) (GWebPluginList*))
                             g_mozilla_plugin_list_get_items_count;
    iface->get_item_title  = (const gchar* (*) (GWebPluginList*, gint))
                             g_mozilla_plugin_list_get_item_title;
    iface->get_item_type   = (const gchar* (*) (GWebPluginList*, gint))
                             g_mozilla_plugin_list_get_item_type;
}

static void
g_mozilla_plugin_list_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    G_MOZILLA_PLUGIN_LIST(instance)->items     = NULL;
    G_MOZILLA_PLUGIN_LIST(instance)->num_items = 0;
}

GType
g_mozilla_plugin_list_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
            {
                sizeof (GMozillaPluginListClass),
                NULL,   /* base_init */
                NULL,   /* base_finalize */
                NULL,   /* class_init */
                NULL,   /* class_finalize */
                NULL,   /* class_data */
                sizeof (GMozillaPluginList),
                0,      /* n_preallocs */
                g_mozilla_plugin_list_instance_init    /* instance_init */
            };
        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_mozilla_plugin_list_interface_init,
                NULL,   /* interface_finalize */
                NULL    /* interface_data */
            };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GMozillaPluginList",
                                       &info, (GTypeFlags) 0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_PLUGIN_LIST,
                                     &iface_info);
    }
    return type;
}

GObject*
g_mozilla_plugin_list_new (void)
{}
