#include "gmozillasupport.h"

ConfigData *config;
gchar *profile_path = NULL;		// mozilla path - probably "~/home/current_user/.mozilla"
gchar *user_home_path;			// relative path to user home directory

// create hidden base directory
void create_config_directory (void)
{

    user_home_path = PR_GetEnv("HOME");
    if (!user_home_path)
    {
        fprintf(stderr, "Failed to get HOME\n");
        exit(1);
    }

    gchar *dir = g_strconcat(user_home_path,"/.mozilla",NULL);

    // this is fine, as it will silently error if it already exists
    mkdir(dir,0755);
    g_free(dir);

    // mozilla config folder - hidden
    profile_path = g_strdup_printf("%s/%s", user_home_path, ".mozilla");
    /* mozilla config folder - hidden */
    gtk_mozilla_set_profile_path (profile_path, "mozilla");

}

// read the current 'mozilla's preferences and configs' into his file: $home/.mozilla/config
void open_and_load_config_file (void)
{
    FILE *config_file;
    gchar *file;
    gchar *line, tmpstr[1024];

    gint i, temp;

    file = g_strconcat(user_home_path,"/.mozilla/config",NULL);
    config = g_new0(ConfigData,1);

    // defaults
    //DEFAULT_HOME_PAGE = g_strconcat(user_home_path, "/", "path_to_mozilla_dir/home_page/OSSO_Sputnik.html", NULL);
    //config->home = DEFAULT_HOME_PAGE;

    config->home = g_strdup("");
    config->use_default_home= 1;
    config->xsize = 700;
    config->ysize = 500;
    config->layout = 0;
    config->mailer = g_strdup("pronto %t %s %c");
    config->max_go = 15;
    config->maxpopupitems = 15;
    config->java = TRUE;
    config->javascript = TRUE;
    config->http_proxy = g_strdup("");
    config->ftp_proxy = g_strdup("");
    config->ftp_proxy_port = g_strdup("");
    config->http_proxy_port = g_strdup("");
    config->ssl_proxy = g_strdup("");
    config->ssl_proxy_port = g_strdup("");
    config->no_proxy_for = g_strdup("localhost");
    config->network_proxy_type = 0;
    config->popup_in_new_window = 0;
    config->blockPopups = 0;
    config->mark_focused = 0;
    config->current_font_size = PREFERENCE_DEFAULT_FONT_SIZE;
    config->memCache = PREFERENCE_DEFAULT_MEM_CACHE;
    config->diskCache = PREFERENCE_DEFAULT_DISK_CACHE;
    config->cookie_behavior= PREFERENCE_DEFAULT_COOKIE_BEHAVIOR;
    config->accept_redirection = PREFERENCE_DEFAULT_ACCEPT_REDIRECTION;
    config->loadImages = PREFERENCE_DEFAULT_LOAD_IMAGE_CONTROL;
    config->frames_enabled = 1;
    config->web_suffix = g_strdup(".com");
    config->enable_ssl2 = 1;

    for (i=0; i < PREFERENCE_LANG_FONT_NUM; i++)
    {
        config->min_font_size [i]= PREFERENCE_DEFAULT_MIN_FONT_SIZE;
    }
    config_file = fopen(file,"r");

    if (config_file == NULL)
    {
        write_config();
    }

    config_file = fopen(file,"r");
    g_return_if_fail (config_file != NULL);
    line = (gchar *)g_malloc(1024);

    while(fgets(line,1024, config_file) != NULL)
    {
        line[strlen(line)-1] = '\0';
        if (g_ascii_strncasecmp(line,"home=",5) == 0)
        {
            config->home = g_strdup(line+5);
            if (g_ascii_strcasecmp(config->home,"") == 0)
            {
                g_free(config->home);
                config->home = g_strdup("about:blank");
            }
        }
        else if (g_ascii_strncasecmp(line,"xsize=",6) == 0)
        {
            config->xsize = atoi(line+6);
        }
        else if (g_ascii_strncasecmp(line,"ysize=",6) == 0)
        {
            config->ysize = atoi(line+6);
        }
        else if (g_ascii_strncasecmp(line,"layout=",7) == 0)
        {
            config->layout = atoi(line+7);
        }
        else if (g_ascii_strncasecmp(line,"mailer=",7) == 0)
        {
            g_free(config->mailer);
            config->mailer = g_strdup(line+7);
        }
        else if (g_ascii_strncasecmp(line,"maxpopup=",9) == 0)
        {
            config->maxpopupitems = atoi(line+9);
        }
        else if (g_ascii_strncasecmp(line,"http_proxy=",11) == 0)
        {
            g_free(config->http_proxy);
            config->http_proxy = g_strdup(line+11);
        }
        else if (g_ascii_strncasecmp(line,"http_proxy_port=",16) == 0)
        {
            g_free(config->http_proxy_port);
            config->http_proxy_port = g_strdup(line+16);
        }
        else if (g_ascii_strncasecmp(line,"ftp_proxy=",10) == 0)
        {
            g_free(config->ftp_proxy);
            config->ftp_proxy = g_strdup(line+10);
        }
        else if (g_ascii_strncasecmp(line,"ftp_proxy_port=",15) == 0)
        {
            g_free(config->ftp_proxy_port);
            config->ftp_proxy_port = g_strdup(line+15);
        }
        else if (g_ascii_strncasecmp(line,"no_proxy_for=",13) == 0)
        {
            g_free(config->no_proxy_for);
            config->no_proxy_for = g_strdup(line+13);
        }
        else if (g_ascii_strncasecmp(line,"ssl_proxy=",10) == 0)
        {
            g_free(config->ssl_proxy);
            config->ssl_proxy = g_strdup(line+10);
        }
        else if (g_ascii_strncasecmp(line,"ssl_proxy_port=",15) == 0)
        {
            g_free(config->ssl_proxy_port);
            config->ssl_proxy_port = g_strdup(line+15);
        }
        else if (g_ascii_strncasecmp(line,"network.proxy.type=",18) == 0)
        {
            config->network_proxy_type = atoi(line+18);
        }
        else if (g_ascii_strncasecmp(line,"max_go=",7) == 0)
        {
            config->max_go = atoi(line+7);
        }
        else if (g_ascii_strncasecmp(line,"popup_in_new_window=",20) == 0)
        {
            config->popup_in_new_window = atoi (line + 20);
        }
        else if (g_ascii_strncasecmp(line,"blockPopups=",12) == 0)
        {
            config->blockPopups = atoi(line+12);
        }
        else if (g_ascii_strncasecmp(line,"fontsize_",9) == 0)
        {
            for (i = 0; i < PREFERENCE_LANG_FONT_NUM; i++)
            {
                g_snprintf (tmpstr, 1024, "%s=", lang_font_item[i]);
                if (g_ascii_strncasecmp(line+9,tmpstr,strlen(tmpstr)))
                    continue;
                config->font_size[i] = atoi(line+9+strlen(tmpstr));
            };
        }
        else if (g_ascii_strncasecmp(line,"min_fontsize_",13) == 0)
        {
            for (i=0; i < PREFERENCE_LANG_FONT_NUM; i++)
            {
                g_snprintf(tmpstr,1024,"%s=",lang_font_item[i]);
                if (g_ascii_strncasecmp(line+13,tmpstr,strlen(tmpstr)))
                    continue;
                config->min_font_size[i] = atoi(line+13+strlen(tmpstr));
            }
        }
        else if (g_ascii_strncasecmp(line,"java=",5) == 0)
        {
            temp = atoi(line+5);
            if (temp)
            {
                config->java = TRUE;
            }
            else
            {
                config->java = FALSE;
            }
        }
        else if (g_ascii_strncasecmp(line,"javascript=",11) == 0)
        {
            temp = atoi(line+11);
            if (temp)
            {
                config->javascript = TRUE;
            }
            else
            {
                config->javascript = FALSE;
            }

        }
        else if (g_ascii_strncasecmp(line,"mark_focused=",13) == 0)
        {
            temp = atoi(line+13);
            if (temp)
            {
                config->mark_focused = TRUE;
            }
            else
            {
                config->mark_focused = FALSE;
            }
        }
        else if (g_ascii_strncasecmp(line,"use_default_home=",17) == 0)
        {
            temp = atoi(line+17);
            if (temp)
            {
                config->use_default_home = TRUE;
            }
            else
            {
                config->use_default_home = FALSE;
            }
        }
        else if (g_ascii_strncasecmp(line,"memCache=",10) == 0)
        {
            config->memCache = atoi(line+10);

        }
        else if (g_ascii_strncasecmp(line,"diskCache=",11) == 0)
        {
            config->diskCache = atoi(line+11);

        }
        else if (g_ascii_strncasecmp(line,"cookie_behavior=",16) == 0)
        {
            config->cookie_behavior = atoi(line+16);

        }
        else if (g_ascii_strncasecmp(line,"load_images=",12) == 0)
        {
            config->loadImages = atoi(line+12);

        }
        else if (g_ascii_strncasecmp(line,"accept_redirection=", 19) == 0)
        {
            temp = atoi(line+19);
            if (temp)
            {
                config->accept_redirection = TRUE;
            }
            else
            {
                config->accept_redirection = FALSE;
            }
        }
        else if (g_ascii_strncasecmp(line,"frames_enabled=", 15) == 0)
        {
            temp = atoi(line+15);
            if (temp)
            {
                config->frames_enabled = TRUE;
            }
            else
            {
                config->frames_enabled = FALSE;
            }
        }
        else if (g_ascii_strncasecmp(line,"web_suffix=", 11) == 0)
        {
            g_free(config->web_suffix);
            config->web_suffix = g_strdup(line+11);
        }
        else if (g_ascii_strncasecmp(line,"enable_ssl2=", 12) == 0)
        {
            temp = atoi(line+12);
            if (temp)
            {
                config->enable_ssl2 = TRUE;
            }
            else
            {
                config->enable_ssl2 = FALSE;
            }
        }
    }
    fflush (config_file);
    fclose(config_file);
    g_free(line);
    g_free(file);
}

// write the current 'mozilla's configs' into his file: $home/.mozilla/config
void write_config ()
{
    FILE *config_file;
    gchar *file;

    file = g_strconcat(user_home_path,"/.mozilla/config",NULL);

    config_file = fopen(file,"w");
    if (config_file == NULL)
    {
        g_error ("Cannot write config file!");
        return ;
    }

    fprintf(config_file,"home=%s\n",config->home);
    fprintf(config_file,"use_default_home=%d\n",config->use_default_home);
    fprintf(config_file,"xsize=%d\n",config->xsize);
    fprintf(config_file,"ysize=%d\n",config->ysize);
    fprintf(config_file,"layout=%d\n",config->layout);
    fprintf(config_file,"mailer=%s\n",config->mailer);
    fprintf(config_file,"maxpopup=%d\n",config->maxpopupitems);
    fprintf(config_file,"http_proxy=%s\n",config->http_proxy);
    fprintf(config_file,"http_proxy_port=%s\n",config->http_proxy_port);
    fprintf(config_file,"ftp_proxy=%s\n",config->ftp_proxy);
    fprintf(config_file,"ftp_proxy_port=%s\n",config->ftp_proxy_port);
    fprintf(config_file,"ssl_proxy=%s\n",config->ssl_proxy);
    fprintf(config_file,"ssl_proxy_port=%s\n",config->ssl_proxy_port);
    fprintf(config_file,"no_proxy_for=%s\n",config->no_proxy_for);
    fprintf(config_file,"network_proxy_type=%d\n",config->network_proxy_type);
    fprintf(config_file,"max_go=%d\n",config->max_go);
    fprintf(config_file,"popup_in_new_window=%d\n",config->popup_in_new_window);
    fprintf(config_file,"mark_focused=%d\n",config->mark_focused);
    fprintf(config_file,"blockPopups=%d\n",config->blockPopups);
    fprintf(config_file,"memCache=%d\n",config->memCache);
    fprintf(config_file,"diskCache=%d\n",config->diskCache);
    fprintf(config_file,"cookie_behavior=%d\n",config->cookie_behavior);
    fprintf(config_file,"accept_redirection=%d\n",config->accept_redirection);
    fprintf(config_file,"load_images=%d\n",config->loadImages);
    fprintf(config_file,"frames_enabled=%d\n",config->frames_enabled);
    fprintf(config_file,"web_suffix=%s\n",config->web_suffix);
    fprintf(config_file,"enable_ssl2=%d\n",config->enable_ssl2);

    if (config->javascript == TRUE)
    {
        fprintf(config_file,"javascript=1\n");
    }
    else
    {
        fprintf(config_file,"javascript=0\n");
    }

    if (config->java == TRUE)
    {
        fprintf(config_file,"java=1\n");
    }
    else
    {
        fprintf(config_file,"java=0\n");
    }

    fflush (config_file);
    fclose(config_file);
    g_free(file);
    set_prefs ();
}

// set mozilla preferences
void set_prefs(void)
{
    gchar *cachedir;
    gtk_mozilla_preference_set_boolean ("security.warn_entering_secure", FALSE);
    gtk_mozilla_preference_set_boolean ("security.warn_leaving_secure", FALSE);
    gtk_mozilla_preference_set_boolean ("security.warn_submit_insecure", FALSE);
    gtk_mozilla_preference_set_boolean ("security.warn_viewing_mixed", FALSE);

    gtk_mozilla_preference_set_boolean("nglayout.widget.gfxscrollbars", TRUE);
    gtk_mozilla_preference_set_boolean ("mozilla.widget.raise-on-setfocus",
                                        FALSE);
    gtk_mozilla_preference_set_boolean ("browser.display.use_system_colors",
                                        FALSE);

    // set default search engine
    gtk_mozilla_preference_set ("keyword.URL",("http://www.google.com/search?q="));
    gtk_mozilla_preference_set_boolean ("keyword.enabled", TRUE);
    gtk_mozilla_preference_set_boolean ("security.checkloaduri", FALSE);

    // while we have no UI
    gtk_mozilla_preference_set_boolean ("wallet.captureForms", FALSE);

    // deactivate mailcap and mime.types support
    gtk_mozilla_preference_set ("helpers.global_mime_types_file", "");
    gtk_mozilla_preference_set ("helpers.global_mailcap_file", "");
    gtk_mozilla_preference_set ("helpers.private_mime_types_file", "");
    gtk_mozilla_preference_set ("helpers.private_mailcap_file", "");

    // dont allow xpi installs from galeon, there are crashes
    gtk_mozilla_preference_set_boolean ("xpinstall.enabled", FALSE);

    // disable sucky XUL ftp view, have nice ns4-like html page instead
    gtk_mozilla_preference_set_boolean ("network.dir.generate_html", TRUE);

    // set the right accept encoding flags
    gtk_mozilla_preference_set ("network.http.accept-encoding" ,
                                "gzip, deflate, compress;q=0.9");

    // mark Focused
    if (config->mark_focused)
    {
        gtk_mozilla_preference_set_boolean ("browser.display.use_focus_colors", TRUE);
        gtk_mozilla_preference_set_int ("browser.display.focus_ring_width", 4); // 0-4
    }
    else
    {
        gtk_mozilla_preference_set_boolean ("browser.display.use_focus_colors", FALSE);
        gtk_mozilla_preference_set_int ("browser.display.focus_ring_width", 1); // 0-4
    }

    // java
    gtk_mozilla_preference_set_boolean ("security.enable_java", config->java);

    // javascript
    gtk_mozilla_preference_set_boolean ("javascript.enabled", config->javascript);

    // highlight on source view
    gtk_mozilla_preference_set_boolean ("browser.view_source.syntax_highlight", TRUE);

    // images handling
    gtk_mozilla_preference_set_boolean ("advanced.always_load_images", false);
    gtk_mozilla_preference_set_int ("network.image.imageBehavior", config->loadImages);

    // cache
    cachedir = g_strconcat(user_home_path,"/.mozilla/",NULL);
    gtk_mozilla_preference_set_int ("browser.cache.disk.capacity",config->diskCache);
    gtk_mozilla_preference_set ("browser.cache.disk.parent_directory", cachedir);
    gtk_mozilla_preference_set_boolean ("browser.cache.disk.enable",TRUE);
    gtk_mozilla_preference_set_boolean ("browser.cache.memory.enable",TRUE);
    gtk_mozilla_preference_set_int ("browser.cache.memory.capacity",config->memCache);
    gtk_mozilla_preference_set_boolean ("network.http.use-cache",TRUE);
    gtk_mozilla_preference_set_boolean ("network.prefetch-next",TRUE);

    // cookies
    // cookie_behavior: 0=Accept, 1=Accept cookies from originating server only, 2=no cookies,
    // 3=use p3p settings (only selectable in the UI if p3p is available)
    if (config->cookie_behavior == 1)
    {
        gtk_mozilla_preference_set_int ("network.cookie.lifetimePolicy", config->cookie_behavior);
        gtk_mozilla_preference_set_boolean ("network.cookie.alwaysAcceptSessionCookies", FALSE);
        gtk_mozilla_preference_set_boolean ("network.cookie.warnAboutCookies", TRUE);
    }
    else
        gtk_mozilla_preference_set_int ("network.cookie.cookieBehavior", config->cookie_behavior);

    // automatic redirection
    if (config->accept_redirection)
        gtk_mozilla_preference_set_boolean ("browser.accept.redirects", TRUE);
    else
        gtk_mozilla_preference_set_boolean ("browser.accept.redirects", FALSE);

    // block popup
    switch (config->blockPopups)
    {
    case 0:
        gtk_mozilla_preference_set_boolean("dom.disable_open_during_load", TRUE);
        //		case 1: read_popup_allowed();
    case 2:
        gtk_mozilla_preference_set_boolean("dom.disable_open_during_load", FALSE);
    }

    // suffix
    gtk_mozilla_preference_set ("browser.fixup.alternate.suffix", config->web_suffix);

    // ssl2
    gtk_mozilla_preference_set_boolean ("security.enable_ssl2", config->enable_ssl2);

    gint network_type = 0;

    if (config->network_proxy_type)
    {
        gtk_mozilla_preference_set("network.proxy.http","");
        gtk_mozilla_preference_set("network.proxy.ssl","");
        gtk_mozilla_preference_set("network.proxy.ftp","");
        gtk_mozilla_preference_set("network.proxy.no_proxies_on", " ");
        gtk_mozilla_preference_set_int("network.proxy.type",network_type);
    }
    else
    {

        if (strlen(config->http_proxy) != 0 && strcmp(config->http_proxy,"") != 0 && strlen(config->http_proxy_port) > 0 && strcmp(config->http_proxy_port,"") != 0)
        {

            gtk_mozilla_preference_set_int ("network.proxy.type", 1);
            network_type=1;
            gtk_mozilla_preference_set ("network.proxy.http", config->http_proxy);
            gtk_mozilla_preference_set_int ("network.proxy.http_port", atoi(config->http_proxy_port));
        }

        if (strlen(config->ftp_proxy) != 0 && strcmp(config->ftp_proxy,"") != 0 && strlen(config->ftp_proxy_port) > 0 && strcmp(config->http_proxy_port,"") != 0)
        {
            if (!network_type)
                gtk_mozilla_preference_set_int("network.proxy.type", 1);
            gtk_mozilla_preference_set ("network.proxy.ftp", config->ftp_proxy);
            gtk_mozilla_preference_set_int ("network.proxy.ftp_port", atoi(config->ftp_proxy_port));
        }

        if (strlen(config->ssl_proxy) != 0 && strcmp(config->ssl_proxy,"") != 0 && strlen(config->ssl_proxy_port) > 0 && strcmp	(config->ssl_proxy_port,"") != 0)
        {
            if (!network_type)
                gtk_mozilla_preference_set_int("network.proxy.type", 1);
            gtk_mozilla_preference_set ("network.proxy.ssl", config->ssl_proxy);
            gtk_mozilla_preference_set_int ("network.proxy.ssl_port", atoi(config->ssl_proxy_port));
        }

        if (strlen(config->no_proxy_for) != 0)
            gtk_mozilla_preference_set("network.proxy.no_proxies_on", config->no_proxy_for);
        else
            gtk_mozilla_preference_set("network.proxy.no_proxies_on", " ");
    }

    // save preferences
    save_prefs ();
}

gboolean save_prefs (void)
{

    nsCOMPtr<nsIPrefService> prefService =
        do_GetService (PREF_ID);
    g_return_val_if_fail (prefService != nsnull, FALSE);
    if (prefService == nsnull)
        return FALSE;
    nsresult rv = prefService->SavePrefFile (nsnull);
    return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

