/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillatransferitem.h: Mozilla Transfer Item Interface Implementation
  * @version $Rev: 2162 $
  * @date $LastChangedDate: 2005-11-10 23:43:16 +0200 (Thu, 10 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include "gmozillatransferitem.h"
#include "gtkmozilla.h"
//#include "gmozillaenginemarshal.h"
#include "gtkmozembedmarshal.h"


extern GMozillaTransferItem *gTransferItemTemp;

static void
g_mozilla_transfer_item_destroy (GMozillaTransferItem *self)
{
    //printf("g_mozilla_transfer_item_destroy\n");

    if (self->item)
    {
        gtk_object_destroy(GTK_OBJECT(self->item));
        self->item = NULL;
    }
}

static void
g_mozilla_transfer_item_start (GMozillaTransferItem *self)
{
    //g_print("g_mozilla_transfer_item_start: %p\n", download->transferItem);
    g_signal_emit_by_name(self, G_WEB_TRANSFER_ITEM_SIGNAL_STARTED, self);
}

static void
g_mozilla_transfer_item_stop (GMozillaTransferItem *self)
{
    if (self->request)
    {
        self->request->Suspend ();
        self->isPaused = TRUE;
    }
}

static void
g_mozilla_transfer_item_continue (GMozillaTransferItem *self)
{
    if (self->request)
    {
        self->request->Resume ();
        self->isPaused = FALSE;
    }
}

static void
g_mozilla_transfer_item_cancel (GMozillaTransferItem *self)
{
    G_MOZILLA_TRANSFER_ITEM(self)->launcher->SetWebProgressListener(nsnull);
    G_MOZILLA_TRANSFER_ITEM(self)->launcher->Cancel(NS_BINDING_ABORTED);

}

static void
g_mozilla_transfer_item_reload (GMozillaTransferItem *self)
{
    g_print("g_mozilla_transfer_item_reload\n");
}

static GObject*
g_mozilla_transfer_item_get_web_item (GMozillaTransferItem *self)
{

    return (GObject *) self;
}

static const gchar*
g_mozilla_transfer_item_get_url (GMozillaTransferItem *self)
{
    //g_print("g_mozilla_transfer_item_get_url\n");
    //return mozilla_get_url();
}

static const gchar*
g_mozilla_transfer_item_get_filename (GMozillaTransferItem *self)
{
    //g_print("g_mozilla_transfer_item_get_filename\n");
    //return mozilla_get_filename();
}

static glong
g_mozilla_transfer_item_get_total_size (GMozillaTransferItem *self)
{
    //g_print("g_mozilla_transfer_item_get_total_size\n");
    //return mozilla_get_total_size();
}

static glong
g_mozilla_transfer_item_get_progress (GMozillaTransferItem *self)
{
    //g_print("g_mozilla_transfer_item_get_progress\n");
    //return mozilla_get_progress();
}

static void
g_mozilla_transfer_item_interface_init (GWebTransferItemIface *iface)
{
    iface->destroy        = (void (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_destroy;
    iface->start          = (void (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_start;
    iface->stop           = (void (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_stop;
    iface->do_continue    = (GWebStatus (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_continue;
    iface->cancel         = (void (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_cancel;
    iface->reload         = (void (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_reload;
    iface->get_web_item   = (GObject* (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_get_web_item;
    iface->get_url        = (const gchar* (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_get_url;
    iface->get_filename   = (const gchar* (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_get_filename;
    iface->get_total_size = (glong (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_get_total_size;
    iface->get_progress   = (glong (*) (GWebTransferItem*))
                            g_mozilla_transfer_item_get_progress;
}

static void
g_mozilla_transfer_item_instance_init (GTypeInstance *instance,
                                       gpointer g_class)
{
    G_MOZILLA_TRANSFER_ITEM(instance)->item = NULL;
}

static void
transfer_item_finalize (GObject *object)
{}

static void
transfer_item_class_init (GMozillaTransferItem *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    object_class->finalize = transfer_item_finalize;

    transfer_item_signals[TRANSFER_ITEM_STARTED_SIGNAL] =
        g_signal_new(G_WEB_TRANSFER_ITEM_SIGNAL_STARTED,
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GMozillaTransferItemClass, started),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);
    transfer_item_signals[TRANSFER_ITEM_COMPLETED_SIGNAL] =
        g_signal_new (G_WEB_TRANSFER_ITEM_SIGNAL_COMPLETED,
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (GMozillaTransferItemClass, completed),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    transfer_item_signals[TRANSFER_ITEM_ERROR_SIGNAL] =
        g_signal_new (G_WEB_TRANSFER_ITEM_SIGNAL_ERROR,
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (GMozillaTransferItemClass, error),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    transfer_item_signals[TRANSFER_ITEM_PROGRESS_SIGNAL] =
        g_signal_new (G_WEB_TRANSFER_ITEM_SIGNAL_PROGRESS,
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (GMozillaTransferItemClass, progress),
                      NULL, NULL,
                      //g_mozilla_engine_marshal_VOID__ULONG_ULONG_ULONG,
                      gtk_marshal_VOID__ULONG_ULONG_ULONG,
                      G_TYPE_NONE,
                      3,
                      G_TYPE_ULONG, G_TYPE_ULONG, G_TYPE_ULONG);
}

GType
g_mozilla_transfer_item_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
            {
                sizeof (GMozillaTransferItemClass),
                NULL,   /* base_init */
                (GBaseFinalizeFunc) transfer_item_finalize,   /* base_finalize */
                (GClassInitFunc) transfer_item_class_init,   /* class_init */
                NULL,   /* class_finalize */
                NULL,   /* class_data */
                sizeof (GMozillaTransferItem),
                0,      /* n_preallocs */
                (GInstanceInitFunc) g_mozilla_transfer_item_instance_init    /* instance_init */
            };
        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_mozilla_transfer_item_interface_init,
                NULL,   /* interface_finalize */
                NULL    /* interface_data */
            };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GMozillaTransferItem",
                                       &info, (GTypeFlags)0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_TRANSFER_ITEM,
                                     &iface_info);
    }
    return type;
}

GObject*
g_mozilla_transfer_item_new (void)
{
    //g_print("g_mozilla_transfer_item_new\n");
    return G_OBJECT(GET_NEW);
}

GObject*
g_mozilla_transfer_item_new_with_item (GObject *item)
{
    //g_print("g_mozilla_transfer_item_new_with_item\n");
    GObject* instance = G_OBJECT(GET_NEW);
    G_MOZILLA_TRANSFER_ITEM(instance)->item = item;

    return instance;
}

GObject*
g_mozilla_transfer_item_new_with_url (const gchar *url,
                                      const gchar *filename)
{
    gTransferItemTemp->server = url;
    gTransferItemTemp->file_name = filename;

    // takes the address of the transferItem created in engine and pass through the layers until the UI.
    GObject *instance = G_OBJECT(gTransferItemTemp);

    return instance;
}
