/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillatransferitemlist.c: Mozilla Transfer Item List Interface Implementation
  * @version $Rev: 2162 $
  * @date $LastChangedDate: 2005-11-10 23:43:16 +0200 (Thu, 10 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include <gwebtransferitem.h>
#include "gmozillatransferitemlist.h"

/* GMozillaTransferItemList */

static void
g_mozilla_transfer_item_list_destroy (GMozillaTransferItemList *self)
{}

static GObject**
g_mozilla_transfer_item_list_get_items (GMozillaTransferItemList *self)
{
    return G_MOZILLA_TRANSFER_ITEM_LIST(self)->items;
}

static gint
g_mozilla_transfer_item_list_get_items_count (GMozillaTransferItemList *self)
{
    return G_MOZILLA_TRANSFER_ITEM_LIST(self)->num_items;
}

static void
g_mozilla_transfer_item_list_interface_init (GWebTransferItemListIface *iface)
{
    iface->destroy         = (void (*) (GWebTransferItemList*))
                             g_mozilla_transfer_item_list_destroy;
    iface->get_items       = (GObject** (*) (GWebTransferItemList*))
                             g_mozilla_transfer_item_list_get_items;
    iface->get_items_count = (gint (*) (GWebTransferItemList*))
                             g_mozilla_transfer_item_list_get_items_count;
}

static void
g_mozilla_transfer_item_list_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    G_MOZILLA_TRANSFER_ITEM_LIST(instance)->items     = NULL;
    G_MOZILLA_TRANSFER_ITEM_LIST(instance)->num_items = 0;
}

GType
g_mozilla_transfer_item_list_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
            {
                sizeof (GMozillaTransferItemListClass),
                NULL,   /* base_init */
                NULL,   /* base_finalize */
                NULL,   /* class_init */
                NULL,   /* class_finalize */
                NULL,   /* class_data */
                sizeof (GMozillaTransferItemList),
                0,      /* n_preallocs */
                g_mozilla_transfer_item_list_instance_init    /* instance_init */
            };
        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_mozilla_transfer_item_list_interface_init,
                NULL,   /* interface_finalize */
                NULL    /* interface_data */
            };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GMozillaTransferItemList",
                                       &info, (GTypeFlags) 0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_TRANSFER_ITEM_LIST,
                                     &iface_info);
    }
    return type;
}

#define GET_NEW (g_object_new(g_mozilla_transfer_item_list_get_type(), NULL))

GObject*
g_mozilla_transfer_item_list_new (void)
{
    g_print("g_mozilla_transfer_item_list_new\n");
    return G_OBJECT(GET_NEW);
}
