/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gmozillaweb.c: Mozilla Web Objects Interface Implementation
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 09:35:49 -0400 (Wed, 02 Nov 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "gmozillaweb.h"
#include "gmozillaengine.h"
#include "gmozillatransferitem.h"
#include "gmozillatransferitemlist.h"
#include "gmozillahistorylist.h"
#include "gmozillapluginlist.h"
#include "gtkmozilla.h"
#include "gmozillasupport.h"
#ifdef USE_OSSOLOG
#include <osso-log.h>
#endif

extern ConfigData *config;

static void
g_mozilla_web_show_error (const char *message)
{
#ifdef USE_OSSOLOG
    ULOG_ERR(message);
#endif
}

static void
g_mozilla_web_destroy (GMozillaWeb *self)
{}

static GWebEngine*
g_mozilla_web_new_web_engine_window (GMozillaWeb *self)
{

    return G_WEB_ENGINE(g_mozilla_engine_new());
}

static GWebEngine*
g_mozilla_web_new_web_engine_window_with_context (GMozillaWeb *self, gpointer context, gboolean leave_tracks, gboolean without_history)
{
    /* THIS IS AN ATTEMPTIVE OF FIXING IT :)*/
    return G_WEB_ENGINE(g_mozilla_engine_new_with_context (context));
}

static GWebTransferItem*
g_mozilla_web_new_web_transfer_item (GMozillaWeb *self, const gchar* url, const gchar* filename)
{
    return G_WEB_TRANSFER_ITEM(g_mozilla_transfer_item_new_with_url(url, filename));
}

static GWebTransferItem*
g_mozilla_web_initialize_web_transfer_item (GMozillaWeb *self, GObject *web_transfer_item)
{
    return G_WEB_TRANSFER_ITEM(g_mozilla_transfer_item_new_with_item(web_transfer_item));
}

static GWebTransferItemList*
g_mozilla_web_get_web_transfer_item_list (GMozillaWeb *self)
{
    return G_WEB_TRANSFER_ITEM_LIST(g_mozilla_transfer_item_list_new());
}

void
g_mozilla_web_set_home_page (GMozillaWeb *self, const gchar* home_page)
{
    config->home=(gchar*)home_page;
    write_config();
}

static gchar*
g_mozilla_web_get_home_page (GMozillaWeb *self)
{
    /*gchar *home_page;
    self->global = cur_engine->engine;
    home_page = gtk_mozilla_get_home_page (GTK_WIDGET(self->global));
    return home_page;*/
    return config->home;
}


static gint
g_mozilla_web_get_maximum_cache_size (GMozillaWeb *self)
{}

static gchar*
g_mozilla_web_get_default_home_page (GMozillaWeb *self)
{
    return config->home;
}

void
g_mozilla_web_home_page_set (GMozillaWeb *self)
{
    write_config();
}

static void
g_mozilla_web_set_frame_borders_enabled (GMozillaWeb *self, gboolean enabled)
{}

static gboolean
g_mozilla_web_get_frame_borders_enabled (GMozillaWeb *self)
{}

void
g_mozilla_web_set_location_completion_suffixes (GMozillaWeb *self, const gchar* suffixes)
{
    config->web_suffix = (gchar*)suffixes;
    write_config();
}

static gchar*
g_mozilla_web_get_location_completion_suffixes (GMozillaWeb *self)
{
    return config->web_suffix;
}

static GWebStatus
g_mozilla_web_set_plugins_enabled (GMozillaWeb *self, gboolean enabled)
{}

void
g_mozilla_web_set_ssl2_support (GMozillaWeb *self, gboolean enabled)
{
    config->enable_ssl2=enabled;
    write_config();
}

static gboolean
g_mozilla_web_get_ssl2_support (GMozillaWeb *self)
{
    return config->enable_ssl2;
}

static gboolean
g_mozilla_web_get_plugins_enabled (GMozillaWeb *self)
{}

void
g_mozilla_web_set_js_enabled (GMozillaWeb *self, gboolean enabled)
{
    config->javascript=enabled;
    write_config();
}

static gboolean
g_mozilla_web_get_js_enabled (GMozillaWeb *self)
{
    return config->javascript;
}

static void
g_mozilla_web_show_memory_cache_error (const char *message)
{
#ifdef USE_OSSOLOG
    ULOG_ERR(message);
#endif

    g_print("[CACHE] %s\n",message);
}

static void
g_mozilla_web_set_memory_cache (GMozillaWeb *self, GMemoryCache acceptance)
{
    gint automatic = 0;

    switch (acceptance)
    {
    case G_MEMORY_CACHE_AUTOMATIC:
        automatic = 1;
        config->memCache = 256;
        break;
    case G_MEMORY_CACHE_OFF:
        config->memCache = 0;
        break;
    case G_MEMORY_CACHE_SMALL:
        config->memCache = 512;
        break;
    case G_MEMORY_CACHE_MEDIUM:
        config->memCache = 1024;
        break;
    case G_MEMORY_CACHE_DEFAULT:
        config->memCache = 2048;
        break;
    case G_MEMORY_CACHE_LARGE:
        config->memCache = 4096;
        break;
    default:
        break;
    }
    //config->memCache = size;
    write_config();
    if (!automatic)
        if (!gtk_mozilla_preference_set_int ("browser.cache.memory.capacity", config->memCache))
            g_mozilla_web_show_memory_cache_error ("Error! It was not possible to set the cache size.");
}

static GMemoryCache
g_mozilla_web_get_memory_cache (GMozillaWeb *self)
{
    if (!gtk_mozilla_preference_get_int ("browser.cache.memory.capacity", &config->memCache))
        g_mozilla_web_show_memory_cache_error ("Error! It was not possible to get the cache size.");

    switch (config->memCache)
    {
    case 0:
        return G_MEMORY_CACHE_OFF;
        break;
    case 512:
        return G_MEMORY_CACHE_SMALL;
        break;
    case 1024:
        return G_MEMORY_CACHE_MEDIUM;
        break;
    case 2048:
        return G_MEMORY_CACHE_DEFAULT;
        break;
    case 4096:
        return G_MEMORY_CACHE_LARGE;
        break;
    default:
        config->memCache = 256;
        return G_MEMORY_CACHE_AUTOMATIC;
        break;
    }

}

#define MAXIMUM_GLOBAL_HISTORY 50
static void
g_mozilla_web_reset_cache_history_length (GMozillaWeb *self)
{
    return;
}

static GWebStatus
g_mozilla_web_set_cache_size (GMozillaWeb *self, gint size)
{
    if (gtk_mozilla_preference_set_int ("browser.cache.memory.capacity", config->memCache))
        return (GWebStatus)FALSE;
    else
        return (GWebStatus)TRUE;
}

static gint
g_mozilla_web_get_cache_size (GMozillaWeb *self)
{
    if (gtk_mozilla_preference_get_int ("browser.cache.memory.capacity", &config->memCache))
        return config->memCache;
    else
        return -1;
}

static GWebStatus
g_mozilla_web_set_browse_offline (GMozillaWeb *self, gboolean offline)
{
    if (gtk_mozilla_preference_set_boolean("network.online",offline))
    {
        if (offline)
            g_print("[OFFLINE] Mozilla is online\n");
        else
            g_print("[OFFLINE] Mozilla is offline\n");
        return (GWebStatus)TRUE;
    }
    else
        return (GWebStatus)FALSE;
}

static gboolean
g_mozilla_web_get_browse_offline (GMozillaWeb *self)
{
    gboolean offline=TRUE;  //Mozilla is online
    gtk_mozilla_preference_get_boolean("network.online",&offline);
    return offline;
}

static GWebStatus
g_mozilla_web_set_automatic_redirection_enabled (GMozillaWeb *self, gboolean enabled)
{
    //return (GWebStatus) gtk_mozilla_prefs_set_int(PREF_CLIENT_PULL_ENABLED, (gint) enabled);
}

static gboolean
g_mozilla_web_get_automatic_redirection_enabled (GMozillaWeb *self)
{/*
            gint enabled=0;
            gtk_mozilla_prefs_get_int(PREF_CLIENT_PULL_ENABLED, &enabled);
            return (gboolean) enabled;*/
}

void
g_mozilla_web_set_load_images (GMozillaWeb *self, GWebEngineImagePolicy level)
{
    switch (level)
    {
    case G_WEBENGINE_POLICY_NO_IMAGES:
        config->loadImages=2;
        break;
    case G_WEBENGINE_POLICY_LOADED_IMAGES:
        config->loadImages=1;
        break;
    case G_WEBENGINE_POLICY_ALL_IMAGES:
        config->loadImages=0;
        break;
    }
    write_config();
}

static GWebEngineImagePolicy
g_mozilla_web_get_load_images (GMozillaWeb *self)
{
    switch (config->loadImages)
    {
    case 0:
        return G_WEBENGINE_POLICY_ALL_IMAGES;
        break;
    case 1:
        return G_WEBENGINE_POLICY_LOADED_IMAGES;
        break;
    case 2:
        return G_WEBENGINE_POLICY_NO_IMAGES;
        break;
    default:
        return G_WEBENGINE_POLICY_ALL_IMAGES;
    }
}

static void
g_mozilla_web_disconnect (GMozillaWeb *self)
{}

static GWebStatus
g_mozilla_web_clear_cache (GMozillaWeb *self)
{
    //guint diskCache = 2;
    guint memoryCache = 1;
    return (GWebStatus) gtk_mozilla_clear_cache(memoryCache);
}

static GWebPluginList*
g_mozilla_web_get_plugins_list (GMozillaWeb *self)
{
    return G_WEB_PLUGIN_LIST(g_mozilla_plugin_list_new());
}

static GWebHistoryList*
g_mozilla_web_get_history_list (GMozillaWeb *self)
{
    return G_WEB_HISTORY_LIST(g_mozilla_history_list_new ());
}

static GWebStatus
g_mozilla_web_set_new_window_loads (GMozillaWeb *self, gboolean loads)
{
    //return (GWebStatus) gtk_mozilla_prefs_set_application_int("Load Home Page", (gint) loads);
}

static gboolean
g_mozilla_web_get_new_window_loads (GMozillaWeb *self)
{}

static void
g_mozilla_web_set_cookie_acceptance (GMozillaWeb *self, GCookieAcceptance acceptance)
{
    gint confirm    = 0;
    gint mode       = 3;
    gint path_err   = 1;
    gint session    = 0;

    if (gtk_mozilla_preference_set_int ("network.cookie.cookieBehavior", (gint) acceptance) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Cookie acceptance settings failed.");
    }

    switch (acceptance)
    {
    case G_ACCEPT_COOKIE_ALWAYS:
        session   = 1;
        break;
    case G_ACCEPT_COOKIE_NEVER:
        mode      = 0;
        break;
    default:
        confirm   = 1;
        path_err  = 0;
        break;
    }

    if (gtk_mozilla_preference_set_int(G_WEB_SIGNAL_ASK_COOKIE          , confirm  ) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Ask For Cookie Confirmation settings failed.");
    }

    if (gtk_mozilla_preference_set_int(G_WEB_SIGNAL_ASK_COOKIE         ,   mode     ) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Enable Cookies mode settings failed");
    }
    if (gtk_mozilla_preference_set_int(G_WEB_SIGNAL_ASK_COOKIE, 1        ) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Do Not Show Cookie Domain Errors settings failed.");
    }
    if (gtk_mozilla_preference_set_int(G_WEB_SIGNAL_ASK_COOKIE , path_err ) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Display warning for cookies with illegal path settings failed.");
    }
    if (gtk_mozilla_preference_set_int(G_WEB_SIGNAL_ASK_COOKIE   , session  ) != G_WEB_STATUS_OK)
    {
        g_mozilla_web_show_error("Error! Accept cookies only for a session settings failed.");
    }
}

static GCookieAcceptance
g_mozilla_web_get_cookie_acceptance (GMozillaWeb *self)
{
    gint cookie_acceptance=0;
    cookie_acceptance = gtk_mozilla_preference_get_int("network.cookie.cookieBehavior",&cookie_acceptance);

    return (GCookieAcceptance)cookie_acceptance;
}

static GSList*
g_mozilla_web_get_cookies_list (GMozillaWeb *self)
{
    //return G_WEB_COOKIE_LIST(g_mozilla_cookie_list_new ());
    return  gtk_mozilla_get_cookie_list();
}

static void
g_mozilla_web_set_popup_acceptance (GMozillaWeb *self, GPopupAcceptance acceptance)
{
    switch (acceptance)
    {
    case G_ACCEPT_POPUP_ALWAYS:
        config->blockPopups=0;
        break;
    case G_ACCEPT_POPUP_PROMPT:
        // Needs prompt stuff
        config->blockPopups=1;
        GMOZ_LOG_NOT_IMP();
        break;
    case G_ACCEPT_POPUP_NEVER:
        config->blockPopups=1;
        break;
    }
    write_config();
}

static GPopupAcceptance
g_mozilla_web_get_popup_acceptance (GMozillaWeb *self)
{

    switch (config->blockPopups)
    {
    case 0:
        return G_ACCEPT_POPUP_ALWAYS;
        break;
    case 1:
        return G_ACCEPT_POPUP_NEVER;
        break;
    default:
        return G_ACCEPT_POPUP_ALWAYS;
    }
}

static GWebStatus
g_mozilla_web_set_language (GMozillaWeb *self, const gchar* language)
{/*
            GWebStatus status = (GWebStatus) gtk_mozilla_prefs_commit();    
            if (gtk_mozilla_web_save_global_history() != GTK_MOZILLA_STATUS_OK) {	    
        		g_mozilla_web_show_error ("Error! Save global history failed.");	    
            }
            gtk_mozilla_reclaim_memory(TRUE);
            gtk_mozilla_suspend(TRUE);
            return status;*/
}

static gchar*
g_mozilla_web_get_language (GMozillaWeb *self)
{/*
            GWebStatus status = (GWebStatus) gtk_mozilla_prefs_commit();    
            if (gtk_mozilla_web_save_global_history() != GTK_MOZILLA_STATUS_OK) {	    
        		g_mozilla_web_show_error ("Error! Save global history failed.");	    
            }
            gtk_mozilla_reclaim_memory(TRUE);
            gtk_mozilla_suspend(TRUE); */
    return NULL;
}

static GWebStatus
g_mozilla_web_suspend (GMozillaWeb *self)
{/*
            GWebStatus status = (GWebStatus) gtk_mozilla_prefs_commit();    
            if (gtk_mozilla_web_save_global_history() != GTK_MOZILLA_STATUS_OK) {	    
        		g_mozilla_web_show_error ("Error! Save global history failed.");	    
            }
            gtk_mozilla_reclaim_memory(TRUE);
            gtk_mozilla_suspend(TRUE);
            return status;*/
}

static gboolean
g_mozilla_web_suspended (GMozillaWeb *self)
{}

static void
g_mozilla_web_resume (GMozillaWeb *self)
{
    //gtk_mozilla_suspend(FALSE);
}

static void
g_mozilla_web_set_ua_string (GMozillaWeb *self, gchar *ua_string)
{}

static void
g_mozilla_web_enable_url_loading (GMozillaWeb *self, gboolean enable)
{}

static GWebStatus
g_mozilla_web_store_transfers (GMozillaWeb *self)
{}

static GWebStatus
g_mozilla_web_restore_transfers (GMozillaWeb *self)
{}

static GWebStatus
g_mozilla_web_delete_all_cookies (GMozillaWeb *self)
{
    //return G_MOZILLA_COOKIE_LIST(G_MOZILLA_WEB(self)->global);
    //return (GWebStatus)gtk_mozilla_web_delete_all_cookies();
    GSList *cookiesList = NULL;
    cookiesList = gtk_mozilla_get_cookie_list();
    return (GWebStatus) gtk_mozilla_delete_all_cookies(cookiesList);
}

static GWebStatus
g_mozilla_web_confirm_cookie (GMozillaWeb *self, gpointer context, gboolean accept, GWebCookieAction action)
{
    //return (GWebStatus) gtk_mozilla_web_cookie_confirm(context, accept, action);
}

static GObject*
g_mozilla_web_get_global (GMozillaWeb *self)
{
    return G_OBJECT(G_MOZILLA_WEB(self)->global);
}

static GWebCertificateMessage
g_mozilla_web_get_certificate_enum_message (GMozillaWeb *self,
        GObject *context)
{}

static GWebCertificateReaction
g_mozilla_web_perform_crtificate_context_action (GMozillaWeb *self,
        GObject *context,
        GWebCertificateAction action,
        GObject *cert)
{}

static GWebStatus
g_mozilla_web_get_certificate_context_iter (GMozillaWeb *self,
        GObject* context,
        GObject **iterator)
{}

static GObject*
g_mozilla_web_get_certificate_iter (GMozillaWeb *self, GObject** iterator)
{}

static void
g_mozilla_web_notify_plugins (GMozillaWeb *self, gint event)
{}

static void
g_mozilla_web_interface_init (GWebIface *iface)
{
    iface->destroy                            = (void (*) (GWeb*))
            g_mozilla_web_destroy;
    iface->new_web_engine_window              = (GWebEngine* (*) (GWeb*))
            g_mozilla_web_new_web_engine_window;
    iface->new_web_engine_window_with_context = (GWebEngine* (*) (GWeb*, gpointer, gboolean, gboolean))
            g_mozilla_web_new_web_engine_window_with_context;
    iface->new_web_transfer_item              = (GWebTransferItem* (*) (GWeb*, const gchar*, const gchar*))
            g_mozilla_web_new_web_transfer_item;
    iface->initialize_web_transfer_item       = (GWebTransferItem* (*) (GWeb*, GObject*))
            g_mozilla_web_initialize_web_transfer_item;
    iface->get_web_transfer_item_list         = (GWebTransferItemList* (*) (GWeb*))
            g_mozilla_web_get_web_transfer_item_list;
    iface->set_home_page                      = (GWebStatus (*) (GWeb*, const gchar*))
            g_mozilla_web_set_home_page;
    iface->get_home_page                      = (gchar* (*) (GWeb*))
            g_mozilla_web_get_home_page;
    iface->set_language                       = (GWebStatus (*) (GWeb*, const gchar*))
            g_mozilla_web_set_language;
    iface->get_language                       = (gchar* (*) (GWeb*))
            g_mozilla_web_get_language;
    iface->reset_cache_history_length         = (void (*) (GWeb*))
            g_mozilla_web_reset_cache_history_length;
    iface->get_maximum_cache_size             = (gint (*) (GWeb*))
            g_mozilla_web_get_maximum_cache_size;
    iface->get_default_home_page              = (gchar* (*) (GWeb*))
            g_mozilla_web_get_default_home_page;
    iface->home_page_set                      = (gboolean (*) (GWeb*))
            g_mozilla_web_home_page_set;
    iface->set_frame_borders_enabled          = (void (*) (GWeb*, gboolean))
            g_mozilla_web_set_frame_borders_enabled;
    iface->get_frame_borders_enabled          = (gboolean (*) (GWeb*))
            g_mozilla_web_get_frame_borders_enabled;
    iface->set_location_completion_suffixes   = (GWebStatus (*) (GWeb*, const gchar*))
            g_mozilla_web_set_location_completion_suffixes;
    iface->get_location_completion_suffixes   = (gchar* (*) (GWeb*))
            g_mozilla_web_get_location_completion_suffixes;
    iface->set_plugins_enabled                = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_plugins_enabled;
    iface->set_ssl2_support                   = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_ssl2_support;
    iface->get_ssl2_support                   = (gboolean (*) (GWeb*))
            g_mozilla_web_get_ssl2_support;
    iface->get_plugins_enabled                = (gboolean (*) (GWeb*))
            g_mozilla_web_get_plugins_enabled;
    iface->set_js_enabled                     = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_js_enabled;
    iface->get_js_enabled                     = (gboolean (*) (GWeb*))
            g_mozilla_web_get_js_enabled;
    iface->set_memory_cache                   = (void (*) (GWeb*, GMemoryCache))
            g_mozilla_web_set_memory_cache;
    iface->get_memory_cache                   = (GMemoryCache (*) (GWeb*))
            g_mozilla_web_get_memory_cache;
    iface->set_cache_size                     = (GWebStatus (*) (GWeb*, gint))
            g_mozilla_web_set_cache_size;
    iface->get_cache_size                     = (gint (*) (GWeb*))
            g_mozilla_web_get_cache_size;
    iface->set_browse_offline                 = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_browse_offline;
    iface->get_browse_offline                 = (gboolean (*) (GWeb*))
            g_mozilla_web_get_browse_offline;
    iface->set_automatic_redirection_enabled  = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_automatic_redirection_enabled;
    iface->get_automatic_redirection_enabled  = (gboolean (*) (GWeb*))
            g_mozilla_web_get_automatic_redirection_enabled;
    iface->set_load_images                    = (GWebStatus (*) (GWeb*, GWebEngineImagePolicy))
            g_mozilla_web_set_load_images;
    iface->get_load_images                    = (GWebEngineImagePolicy (*) (GWeb*))
            g_mozilla_web_get_load_images;
    iface->disconnect                         = (void (*) (GWeb*))
            g_mozilla_web_disconnect;
    iface->clear_cache                        = (GWebStatus (*) (GWeb*))
            g_mozilla_web_clear_cache;
    iface->get_plugins_list                   = (GWebPluginList* (*) (GWeb*))
            g_mozilla_web_get_plugins_list;
    iface->get_history_list                   = (GWebHistoryList* (*) (GWeb*))
            g_mozilla_web_get_history_list;
    iface->set_new_window_loads               = (GWebStatus (*) (GWeb*, gboolean))
            g_mozilla_web_set_new_window_loads;
    iface->get_new_window_loads               = (gboolean (*) (GWeb*))
            g_mozilla_web_get_new_window_loads;
    iface->set_cookie_acceptance              = (void (*) (GWeb*, GCookieAcceptance))
            g_mozilla_web_set_cookie_acceptance;
    iface->get_cookie_acceptance              = (GCookieAcceptance (*) (GWeb*))
            g_mozilla_web_get_cookie_acceptance;
    iface->set_popup_acceptance               = (void (*) (GWeb*, GPopupAcceptance))
            g_mozilla_web_set_popup_acceptance;
    iface->get_popup_acceptance               = (GPopupAcceptance (*) (GWeb*))
            g_mozilla_web_get_popup_acceptance;
    iface->suspend                            = (GWebStatus (*) (GWeb*))
            g_mozilla_web_suspend;
    iface->suspended                          = (gboolean (*) (GWeb*))
            g_mozilla_web_suspended;
    iface->resume                             = (void (*) (GWeb*))
            g_mozilla_web_resume;
    iface->set_ua_string                      = (void (*) (GWeb*, gchar*))
            g_mozilla_web_set_ua_string;
    iface->enable_url_loading                 = (void (*) (GWeb*, gboolean))
            g_mozilla_web_enable_url_loading;
    iface->store_transfers                    = (GWebStatus (*) (GWeb*))
            g_mozilla_web_store_transfers;
    iface->restore_transfers                  = (GWebStatus (*) (GWeb*))
            g_mozilla_web_restore_transfers;
    iface->delete_all_cookies                 = (GWebStatus (*) (GWeb*))
            g_mozilla_web_delete_all_cookies;
    iface->confirm_cookie                     = (GWebStatus (*) (GWeb*, gpointer, gboolean, GWebCookieAction))
            g_mozilla_web_confirm_cookie;
    iface->get_global                         = (GObject* (*) (GWeb*))
            g_mozilla_web_get_global;
    iface->get_certificate_enum_message       = (GWebCertificateMessage (*) (GWeb*, GObject*))
            g_mozilla_web_get_certificate_enum_message;
    iface->perform_crtificate_context_action  = (GWebCertificateReaction (*) (GWeb*,
            GObject*,
            GWebCertificateAction,
            GObject*))
            g_mozilla_web_perform_crtificate_context_action;
    iface->get_certificate_context_iter       = (GWebStatus (*) (GWeb*, GObject*, GObject**))
            g_mozilla_web_get_certificate_context_iter;
    iface->get_certificate_iter               = (GObject* (*) (GWeb*, GObject**))
            g_mozilla_web_get_certificate_iter;
    iface->notify_plugins                     = (void (*) (GWeb*, gint))
            g_mozilla_web_notify_plugins;
}


static void
g_mozilla_web_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    G_MOZILLA_WEB(instance)->global = NULL;
}

GType
g_mozilla_web_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
            {
                sizeof (GMozillaWebClass),
                NULL,   /* base_init */
                NULL,   /* base_finalize */
                NULL,   /* class_init */
                NULL,   /* class_finalize */
                NULL,   /* class_data */
                sizeof (GMozillaWeb),
                0,      /* n_preallocs */
                g_mozilla_web_instance_init    /* instance_init */
            };
        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_mozilla_web_interface_init,
                NULL,   /* interface_finalize */
                NULL    /* interface_data */
            };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GMozillaWeb",
                                       &info, (GTypeFlags) 0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB,
                                     &iface_info);
    }
    return type;
}

/*GObject*
g_web_new (void)
{
    //    printf("%s, %i\n", __FUNCTION__, __LINE__);
    GObject* instance = G_OBJECT(g_object_new(g_mozilla_web_get_type(), NULL));
    G_MOZILLA_WEB(instance)->global = instance;
    //    G_MOZILLA_WEB(instance)->global = g_mozilla_engine_new();
    return instance;
}*/


GObject*
g_web_new (void)
{
    GObject* instance = G_OBJECT(g_object_new(g_mozilla_web_get_type(), NULL));
    G_MOZILLA_WEB(instance)->global = G_OBJECT(gtk_mozilla_web_engine_new());
 
    return instance;
 
}

void
g_web_set_env (void)
{
    gchar *ld_lib_path = g_strdup_printf("%s:%s", LD_PATH, getenv ("LD_LIBRARY_PATH"));
    gchar *ld_lib_all_path = g_strdup_printf("%s%s:%s", LD_PATH, "/components", LD_PATH);
    setenv("LD_LIBRARY_PATH", ld_lib_path, 0);
    setenv("MOZILLA_FIVE_HOME", LD_PATH, 0);
    setenv("DYLD_LIBRARY_PATH", LD_PATH, 0);
    setenv("LIBRARY_PATH", ld_lib_all_path, 0);
    setenv("SHLIB_PATH", LD_PATH, 0);
    setenv("LIBPATH", LD_PATH, 0);
    setenv("ADDON_PATH", LD_PATH, 0);
    setenv("MOZ_TOOLKIT", "", 0);
    setenv("moz_debug", "", 0);
    setenv("moz_debugger", "", 0);
    g_free(ld_lib_path);
    g_free(ld_lib_all_path);
}
