/**
  * Mozilla Web Engine Abstraction Layer.
  * @file gtkmozilla.c: GTK Mozilla Functions Implementation
  * @version $Rev: 2093 $
  * @date $LastChangedDate: 2005-11-01 10:22:11 -0400 (Tue, 01 Nov 2005) $
  *
  * Copyright (C) 2005 10LE.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include <osso-log.h>
#include "gtkmozilla.h"

// testing window_request signal
typedef struct _context
{

    GObject *embed;

}
context_struct ;


GtkWidget* gtk_mozilla_web_engine_new()
{

    GtkWidget *engine;

    //engine=gtk_moz_embed_new();
    engine = GTK_WIDGET (g_mozilla_engine_new ());
    return engine;
}


void gtk_mozilla_load_url (GtkWidget *engine, const gchar *url)
{

    gtk_moz_embed_load_url (GTK_MOZ_EMBED(engine), url);
}

GtkWidget* gtk_mozilla_web_engine_new_with_context (gpointer context)
{


    GtkWidget *engine;
    //engine=gtk_moz_embed_new();
    engine = GTK_WIDGET (g_mozilla_engine_new ());

    // testing window_request signal
    if (context != 0)
        ((context_struct *) context)->embed = G_OBJECT (engine);

    return engine;
}


void gtk_mozilla_go_back (GtkWidget *engine)
{

    gtk_moz_embed_go_back (GTK_MOZ_EMBED(engine));

}


void gtk_mozilla_go_forward (GtkWidget *engine)
{

    gtk_moz_embed_go_forward (GTK_MOZ_EMBED(engine));

}


gboolean gtk_mozilla_can_go_back (GtkWidget *engine)
{

    return (gtk_moz_embed_can_go_back (GTK_MOZ_EMBED(engine)));
}


gboolean gtk_mozilla_can_go_forward (GtkWidget *engine)
{

    return (gtk_moz_embed_can_go_forward (GTK_MOZ_EMBED(engine)));
}


void gtk_mozilla_stop_load (GtkWidget *engine)
{

    gtk_moz_embed_stop_load (GTK_MOZ_EMBED(engine));

}


void gtk_mozilla_reload (GtkWidget *engine)
{

    gtk_moz_embed_reload (GTK_MOZ_EMBED(engine), GTK_MOZ_EMBED_FLAG_RELOADNORMAL);

}


gchar *gtk_mozilla_get_home_page (GtkWidget *engine)
{

    /* FIX ME */
    return "";

}


gchar *gtk_mozilla_get_frame_title (GtkWidget *engine)
{

    return gtk_moz_embed_get_title (GTK_MOZ_EMBED(engine));

}

gchar*
latin1_to_utf8 (const char *text)
{
    GString *str;
    const gchar *p;

    str = g_string_new ("");

    p = text;
    while (*p)
    {

        g_string_append_unichar (str, *p);
        ++p;
    }

    return g_string_free (str, FALSE); /* return str->str and free str */
}

gboolean gtk_mozilla_preference_set (const char *preference_name, const char *new_value)
{
    g_return_val_if_fail (preference_name != NULL, FALSE);
    g_return_val_if_fail (new_value != NULL, FALSE);

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {

        nsresult rv = pref->SetCharPref (preference_name, new_value);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }
    return (FALSE);
}


gboolean gtk_mozilla_preference_set_boolean (const char *preference_name, gboolean new_boolean_value)
{

    g_return_val_if_fail (preference_name != NULL, FALSE);

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {
        nsresult rv = pref->SetBoolPref (preference_name,  new_boolean_value ? PR_TRUE : PR_FALSE);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }

    return (FALSE);
}

gboolean gtk_mozilla_preference_set_int (const char *preference_name, gint  new_int_value)
{
    g_return_val_if_fail (preference_name != NULL, FALSE);

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {
        nsresult rv = pref->SetIntPref (preference_name, new_int_value);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }

    return (FALSE);
}

gboolean
gtk_mozilla_preference_get_boolean (const char *preference_name, gboolean *value)
{

    g_return_val_if_fail (preference_name != NULL, FALSE);

    nsresult rv;

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {
        nsresult rv = pref->GetBoolPref (preference_name, value);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }

    return (FALSE);
}

gboolean
gtk_mozilla_preference_get_int (const char *preference_name, gint *value)
{

    g_return_val_if_fail (preference_name != NULL, FALSE);

    nsresult rv;

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {
        nsresult rv = pref->GetIntPref (preference_name, value);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }

    return (FALSE);
}

gboolean
gtk_mozilla_preference_get (const char *preference_name, gchar **value)
{

    g_return_val_if_fail (preference_name != NULL, FALSE);

    nsresult rv;

    nsCOMPtr<nsIPref> pref = do_CreateInstance(PREF_ID);

    if (pref)
    {
        nsresult rv = pref->GetCharPref (preference_name, value);
        return ( NS_SUCCEEDED (rv) ? TRUE : FALSE );
    }

    return (FALSE);
}

void gtk_mozilla_set_profile_path(const char *aDir, const char *aName)
{

    gtk_moz_embed_set_profile_path(aDir, aName);

}

/*
void gtk_mozilla_set_zoom_scale (GtkWidget * engine, int zoom_scale) {
 
	nsIWebBrowser *webBrowser = nsnull;
	g_return_if_fail(GTK_MOZ_EMBED(engine) != NULL);
	gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);
		
	nsCOMPtr<nsIDOMWindow> DOMWindow;
	webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
	g_return_if_fail (DOMWindow != NULL);
 
	nsCOMPtr<nsIDOMDocument> doc;
	DOMWindow->GetDocument (getter_AddRefs(doc));
	g_return_if_fail (doc != NULL);
	
	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(doc);
	g_return_if_fail (node != NULL);
 
	nsCOMPtr<nsIZoomer> zoomer = do_CreateInstance("@mozilla.org/zoomer;1");
 
	if (zoomer) {
 
		zoomer->Zoom (node, zoom_scale);
	}
}
*/

void gtk_mozilla_set_zoom_scale (GtkWidget * engine, gint zoom_scale)
{
    const PRUnichar imgLiteral[] =
        { 'i', 'm', 'g', '\0'
        };
    const PRUnichar styleLiteral[] =
        { 's', 't', 'y', 'l', 'e', '\0'
        };
    nsIWebBrowser *webBrowser = nsnull;
    g_return_if_fail(GTK_MOZ_EMBED(engine) != NULL);
    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);

    g_return_if_fail (webBrowser != NULL);

    nsCOMPtr<nsIDOMWindow> DOMWindow;
    webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
    g_return_if_fail (DOMWindow != NULL);

    nsCOMPtr<nsIDOMDocument> doc;
    DOMWindow->GetDocument (getter_AddRefs(doc));
    g_return_if_fail (doc != NULL);

    nsCOMPtr<nsIDOMNodeList> nodeList;
    doc->GetElementsByTagName(nsEmbedString(imgLiteral),
                              getter_AddRefs(nodeList));

    PRUint32 num;
    nodeList->GetLength(&num);


    float curr_zoom = 1;
    if(DOMWindow)
    {
        DOMWindow->GetTextZoom(&curr_zoom);
    }

    float new_zoom = (float)zoom_scale/100.;

    for (PRUint32 i = 0; i < num; i++)
    {
        nsCOMPtr<nsIDOMNode> node;
        nodeList->Item(i, getter_AddRefs(node));
        nsCOMPtr<nsIDOMHTMLImageElement> img;
        img = do_QueryInterface(node);
        PRInt32 width = 0, height = 0;
        img->GetHeight(&height);
        img->GetWidth(&width);
        width = (PRInt32)((float)width * (float)(new_zoom/curr_zoom));
        height = (PRInt32)((float)height * (float)(new_zoom/curr_zoom));
        nsCOMPtr<nsIDOMElement> elm;
        elm = do_QueryInterface(node);
        nsEmbedString style_value;
        gchar *v;
        v = g_strdup_printf("width: %dpx; height: %dpx;",
                            width, height);
        nsEmbedCString cValue(v);
        NS_CStringToUTF16(cValue, NS_CSTRING_ENCODING_UTF8, style_value);
        elm->SetAttribute(nsEmbedString(styleLiteral), style_value);
        g_free(v);
    }


    if(DOMWindow)
    {
        DOMWindow->SetTextZoom(new_zoom);
    }
}


gint gtk_mozilla_get_zoom_scale (GtkWidget * engine)
{
    nsIWebBrowser *webBrowser = nsnull;
    g_return_val_if_fail(GTK_MOZ_EMBED(engine) != NULL, 100);
    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);
    if (webBrowser == NULL)
        return 100;
    nsCOMPtr<nsIDOMWindow> DOMWindow;

    webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
    g_return_val_if_fail (DOMWindow != NULL, 100);
    float curr_zoom = 100;
    if(DOMWindow)
    {
        DOMWindow->GetTextZoom(&curr_zoom);
    }
    return (int)(curr_zoom * 100.);
}

/*
 * g_mozilla_engine_locale_to_unicode: Decodes a string encoded for the current
 * locale into unicode. Used for getting text entered in a GTK+ entry
 * into a form which mozilla can use.
 */
PRUnichar *
gtk_mozilla_engine_locale_to_unicode (const gchar *locStr)
{

    if (locStr == NULL)
    {

        return (NULL);
    }

    nsAutoString autoStr;
    autoStr.AssignWithConversion (locStr);
    PRUnichar *uniStr = ToNewUnicode(autoStr);
    return ( uniStr );
}


gboolean
gtk_mozilla_find_text (GtkWidget * engine, const gchar *exp, PRBool  reverse, PRBool  whole_word, PRBool  case_sensitive, PRBool  restart )
{

    PRUnichar *search_string;

    g_return_val_if_fail( engine != NULL, FALSE);
    nsIWebBrowser *wb = nsnull;
    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED (engine), &wb);
    nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface (wb));
    g_return_val_if_fail( finder != NULL, FALSE);

    search_string = gtk_mozilla_engine_locale_to_unicode (exp);
    finder->SetSearchString (search_string);
    finder->SetFindBackwards (reverse);
    finder->SetWrapFind(TRUE); //DoWrapFind
    finder->SetEntireWord (whole_word);
    finder->SetSearchFrames(TRUE); //SearchInFrames
    finder->SetMatchCase (case_sensitive);
    finder->FindNext (&restart);

    g_free(search_string);
    return ( reverse );
}

nsresult
gtk_mozilla_select_all (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    return clipboard->SelectAll ();
}


nsresult
gtk_mozilla_cut (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    return clipboard->CutSelection ();
}

nsresult
gtk_mozilla_copy (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    return clipboard->CopySelection ();
}

nsresult
gtk_mozilla_paste (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    return clipboard->Paste ();
}

PRBool
gtk_mozilla_can_copy (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    PRBool canCopy;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    clipboard->CanCopySelection (&canCopy);
    return canCopy;
}

PRBool
gtk_mozilla_can_paste (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    PRBool canPaste;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    clipboard->CanPaste (&canPaste);
    return canPaste;
}

PRBool
gtk_mozilla_can_cut (GtkWidget * engine)
{

    nsIWebBrowser *wb = nsnull;
    PRBool canCut;
    gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(engine), &wb);
    nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(wb));
    clipboard->CanCutSelection (&canCut);
    return canCut;
}

PRBool
gtk_mozilla_can_selecttext (GtkWidget * engine)
{
}

gboolean
mozilla_register_components (void)
{
    gboolean ret = TRUE;
    nsresult rv;

    nsCOMPtr<nsIComponentRegistrar> cr;
    rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
    NS_ENSURE_SUCCESS(rv, rv);

    nsCOMPtr<nsIComponentManager> cm;
    NS_GetComponentManager (getter_AddRefs (cm));
    NS_ENSURE_TRUE (cm, FALSE);

    guint numAppComps = G_N_ELEMENTS (sAppComps);

    for (guint i = 0; i < G_N_ELEMENTS (sAppComps); i++)
    {
        nsCOMPtr<nsIGenericFactory> componentFactory;
        rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
                                  &(sAppComps[i]));
        if (NS_FAILED(rv) || !componentFactory)
        {
            g_warning ("Failed to make a factory for %s\n", sAppComps[i].mDescription);

            ret = FALSE;
            continue;  // don't abort registering other components
        }

        rv = cr->RegisterFactory(sAppComps[i].mCID,
                                 sAppComps[i].mDescription,
                                 sAppComps[i].mContractID,
                                 componentFactory);
        if (NS_FAILED(rv))
        {
            g_warning ("Failed to register %s\n", sAppComps[i].mDescription);

            ret = FALSE;
        }

        if (sAppComps[i].mRegisterSelfProc)
        {
            rv = sAppComps[i].mRegisterSelfProc (cm, nsnull, nsnull, nsnull, &sAppComps[i]);

            if (NS_FAILED (rv))
            {
                g_warning ("Failed to register-self for %s\n", sAppComps[i].mDescription);
                ret = FALSE;
            }
        }
    }
    return ret;
}

gchar* gtk_mozilla_get_mime (GtkWidget * engine)
{


    nsIWebBrowser *webBrowser = nsnull;

    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);

    nsCOMPtr<nsIDOMWindow> DOMWindow;
    webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));

    nsCOMPtr<nsIDOMDocument> doc;
    DOMWindow->GetDocument (getter_AddRefs(doc));

    nsCOMPtr<nsIDOMNSDocument> nsDoc = do_QueryInterface(doc);

    nsString mime;
    nsDoc->GetContentType(mime);
    return g_strdup(NS_LossyConvertUTF16toASCII(mime).get());

}

nsresult gtk_mozilla_get_content_viewer (GtkWidget *engine, nsIContentViewer **aViewer)
{
    nsIWebBrowser *webBrowser = nsnull;

    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);
    nsCOMPtr<nsIDocShell> ourDocShell(do_GetInterface(webBrowser));

    return ourDocShell->GetContentViewer(aViewer);
}

gchar *gtk_mozilla_get_encoding (GtkWidget *engine)
{

    char * charset;
    nsIWebBrowser *webBrowser = nsnull;

    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);
    nsCOMPtr<nsIDocCharset> docCharset = do_GetInterface (webBrowser);

    docCharset->GetCharset (&charset);

    return charset;

}

nsresult gtk_mozilla_set_encoding (GtkWidget *engine, const gchar *encoding)
{

    nsCOMPtr<nsIContentViewer> contentViewer;
    gtk_mozilla_get_content_viewer (engine, getter_AddRefs(contentViewer));

    nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);

    nsAutoString mCharset;
    mCharset.AssignWithConversion (encoding);

    return mdv->SetForceCharacterSet(NS_LossyConvertUTF16toASCII(ToNewUnicode(mCharset)));

}

gboolean gtk_mozilla_clear_cache (gint mode)
{

    nsCOMPtr<nsICacheService> CacheService = do_GetService (NS_CACHESERVICE_CONTRACTID);
    if (CacheService)
    {
        CacheService->EvictEntries(mode);
        return FALSE;   //GWebStatus 0 == OK
    }
    else
        return TRUE;
}

GSList* gtk_mozilla_get_cookie_list(void)
{
    GSList *cookies = NULL;
    nsresult result;

    nsCOMPtr<nsICookieManager> cookieManager =
        do_GetService(NS_COOKIEMANAGER_CONTRACTID);

    nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
    result = cookieManager->GetEnumerator(getter_AddRefs(cookieEnumerator));

    g_return_val_if_fail(NS_SUCCEEDED(result), NULL);
    PRBool enumResult;

    for (cookieEnumerator->HasMoreElements(&enumResult) ;
            enumResult == PR_TRUE ;
            cookieEnumerator->HasMoreElements(&enumResult))
    {
        GMozillaCookieList *c;

        nsCOMPtr<nsICookie> nsCookie;
        result = cookieEnumerator->GetNext(getter_AddRefs(nsCookie));
        g_return_val_if_fail(NS_SUCCEEDED(result), NULL);

        c = g_new0(GMozillaCookieList, 1);

        nsCAutoString transfer;

        nsCookie->GetHost(transfer);
        c->domain = g_strdup(transfer.get());
        g_print("Domain = %s \n", c->domain);

        nsCookie->GetName(transfer);
        c->name = g_strdup(transfer.get());
        g_print("Name = %s \n", c->name);

        nsCookie->GetValue(transfer);
        c->value = g_strdup(transfer.get());
        g_print("Value = %s \n", c->value);

        nsCookie->GetPath(transfer);
        if (strchr(c->domain,'.'))
            c->path = g_strdup(g_strconcat("http://*",c->domain,"/",NULL));
        else
            c->path = g_strdup(g_strconcat("http://",c->domain,"/",NULL));
        g_print("Path = %s \n", c->path);

        cookies = g_slist_prepend(cookies, c);
    }
    cookies = g_slist_reverse(cookies);
    return cookies;

}

gint gtk_mozilla_delete_all_cookies (GSList *deletedCookies)
{

    if (!deletedCookies)
    {
        g_print("\nLista de cookies a ser deletada esta vazia\n");
        return 1;
    }
    else
    {
        nsCOMPtr<nsICookieManager> cookieManager =
            do_GetService(NS_COOKIEMANAGER_CONTRACTID);

        cookieManager->RemoveAll();

        g_slist_free (deletedCookies);

        return 0;//False in GWebStatus means OK, as opposed to gboolean in C
    }
}

glong
gtk_mozilla_get_context_menu_type(GtkWidget *engine,
                                  gpointer event, GContextData * ctx_data)
{
    nsresult result;
    nsresult rv;

    nsIWebBrowser *webBrowser = nsnull;
    gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED(engine), &webBrowser);

    nsCOMPtr<nsIDOMWindow> DOMWindow;
    webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));

    nsIDOMMouseEvent *aMouseEvent =  (nsIDOMMouseEvent*)event;
    nsCOMPtr<nsIDOMEventTarget> OriginalTarget;

    nsCOMPtr<nsIDOMNSEvent> aEvent = do_QueryInterface(aMouseEvent);

    if (!aEvent)
        return G_WEBENGINE_POPUPMENU_NOTYPE;

    result = aEvent->GetOriginalTarget(getter_AddRefs(OriginalTarget));
    if (NS_FAILED(result) || !OriginalTarget)
        return G_WEBENGINE_POPUPMENU_NOTYPE;

    nsCOMPtr<nsIDOMNode> OriginalNode = do_QueryInterface(OriginalTarget);

    if (!OriginalNode)
        return G_WEBENGINE_POPUPMENU_NOTYPE;
    GulString nodename;
    OriginalNode->GetNodeName(nodename);
    if (nodename.EqualsLiteral ("xul:thumb") ||
            nodename.EqualsLiteral ("xul:slider") ||
            nodename.EqualsLiteral ("xul:scrollbarbutton"))
        return G_WEBENGINE_POPUPMENU_NOTYPE;



    nsCOMPtr<nsIDOMEventTarget> EventTarget;
    result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
    if (NS_FAILED(result) || !EventTarget)
        return (  G_WEBENGINE_POPUPMENU_NOTYPE );

    nsCOMPtr<nsIDOMNode> node = do_QueryInterface(EventTarget);

    if (!node)
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    nsCOMPtr<nsIDOMDocument> mainDocument;
    DOMWindow->GetDocument (getter_AddRefs(mainDocument));
    g_return_val_if_fail (mainDocument != NULL, G_WEBENGINE_POPUPMENU_NOTYPE);

    nsCOMPtr<nsIDOMDocument> domDoc;
    result = node->GetOwnerDocument(getter_AddRefs(domDoc));
    if (!NS_SUCCEEDED (result) || !domDoc)
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
    if(!doc)
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    nsCOMPtr<nsIDOMNSDocument> nsDoc = do_QueryInterface(domDoc);
    if (!nsDoc)
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    nsIURI *baseURI;
    baseURI = doc->GetBaseURI ();

    if (!baseURI)
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    nsString mime;
    nsDoc->GetContentType(mime);
    if (mime.EqualsLiteral("text/xul"))
        return (G_WEBENGINE_POPUPMENU_NOTYPE);

    PRUint32 flags = G_WEBENGINE_POPUPMENU_NOTYPE;

    if (mainDocument != domDoc)
    {

        flags |= G_WEBENGINE_POPUPMENU_IFRAME;
        nsCOMPtr<nsIDOM3Document> docuri = do_QueryInterface(domDoc);
        nsCString cfuri;
        nsAutoString afuri;
        docuri->GetDocumentURI (afuri);
        cfuri.AssignWithConversion(afuri);
        ctx_data->framehref = ToNewCString(cfuri);
    }

    nsCOMPtr<nsISelection> selection;
    rv = DOMWindow->GetSelection(getter_AddRefs(selection));
    if (NS_SUCCEEDED(rv))
    {
        PRUnichar * selText;
        rv = selection->ToString (&selText);
        if (NS_SUCCEEDED(rv) && selText)
        {
            nsEmbedCString str;
            NS_UTF16ToCString(nsEmbedString(selText),
                              NS_CSTRING_ENCODING_UTF8, str);

            ctx_data->linktext = g_strdup (str.get());
            if (strlen(ctx_data->linktext) > 0)
                flags |= G_WEBENGINE_POPUPMENU_TEXTSELECTION;
            else
            {
                g_free(ctx_data->linktext);
                ctx_data->linktext = NULL;
            }
        }
    }

    PRUint16 type;
    rv = node->GetNodeType(&type);
    if (NS_FAILED(rv))
        return G_WEBENGINE_POPUPMENU_NOTYPE;

    PRBool has_background = PR_FALSE;
    nsCOMPtr<nsIDOMHTMLElement> element  = do_QueryInterface(node);

    if ((nsIDOMNode::ELEMENT_NODE == type) && element)
    {
        GulString uTag;
        rv = element->GetLocalName(uTag);
        if (NS_FAILED(rv))
            return G_WEBENGINE_POPUPMENU_NOTYPE;

        GulCString tag (uTag);
        if (tag.EqualsAsciiIgnoreCase("img"))
        {
            GulString img;
            nsCOMPtr <nsIDOMHTMLImageElement> image =
                do_QueryInterface(node, &rv);
            if (NS_FAILED(rv) || !image)
                return G_WEBENGINE_POPUPMENU_NOTYPE;

            rv = image->GetSrc (img);
            if (NS_FAILED(rv))
                return G_WEBENGINE_POPUPMENU_NOTYPE;

            ctx_data->imghref = ToNewCString(img);

            rv = image->GetAlt (img);
            if (NS_SUCCEEDED(rv))
            {
                ctx_data->linktext = ToNewCString(img);
            }

            int imgwidth, imgheight;
            rv = image->GetWidth (&imgwidth);
            rv = image->GetHeight (&imgheight);

            flags |= G_WEBENGINE_POPUPMENU_IMAGE;
        }
        else if (tag.EqualsAsciiIgnoreCase("input"))
        {
            nsCOMPtr<nsIDOMHTMLInputElement> input = do_QueryInterface (node);
            GulString ginput;
            rv = input->GetType(ginput);
            GulCString gcinput (ginput);
            if (gcinput.EqualsAsciiIgnoreCase("text"))
            {
                flags |= CONTEXT_INPUT;
            }
        }
        else if (tag.EqualsAsciiIgnoreCase("textarea"))
        {
            flags |= CONTEXT_INPUT;
        }
        else if (tag.EqualsAsciiIgnoreCase("object"))
        {
            nsCOMPtr<nsIDOMHTMLObjectElement> object;
            object = do_QueryInterface (node);

            if (!element)
                return G_WEBENGINE_POPUPMENU_NOTYPE;

            GulString value;
            object->GetType(value);
            GulCString cValue = value;
            if (cValue.StartsWith("image/"))
            {
                GulString img;
                rv = object->GetData (img);
                if (NS_FAILED(rv))
                    return G_WEBENGINE_POPUPMENU_NOTYPE;
                flags |= G_WEBENGINE_POPUPMENU_IMAGE;
                ctx_data->imghref = ToNewCString(img);
            }
            else
            {
                return G_WEBENGINE_POPUPMENU_NOTYPE;
            }
        }
        else if (tag.EqualsAsciiIgnoreCase("html"))
        {
            nsCOMPtr<nsIDOMNodeList> nodeList;
            rv = domDoc->GetElementsByTagName (NS_LITERAL_STRING ("body"),
                                               getter_AddRefs (nodeList));
            if (NS_SUCCEEDED (rv) && nodeList)
            {
                nsCOMPtr<nsIDOMNode> bodyNode;
                nodeList->Item (0, getter_AddRefs (bodyNode));
                GulString cssurl;
            }
        }
    }

    while (node)
    {
        nsCOMPtr <nsIDOMElement> dom_elem = do_QueryInterface(node);
        if (dom_elem)
        {

            NS_NAMED_LITERAL_STRING(nspace, "http://www.w3.org/1999/xlink");
            NS_NAMED_LITERAL_STRING(localname_type, "type");

            GulString value;
            dom_elem->GetAttributeNS (nspace, localname_type, value);
            GulCString cValue = value;
            if (cValue.EqualsAsciiIgnoreCase ("simple"))
            {

                NS_NAMED_LITERAL_STRING (localname_href, "href");
                dom_elem->GetAttributeNS (nspace, localname_href, value);
            }

        }

        PRUint16 type;
        rv = node->GetNodeType(&type);
        if (NS_FAILED(rv))
            return G_WEBENGINE_POPUPMENU_NOTYPE;
        element = do_QueryInterface(node);
        if ((nsIDOMNode::ELEMENT_NODE == type) && element)
        {

            GulString uTag;
            rv = element->GetLocalName(uTag);
            if (NS_FAILED(rv))
                return G_WEBENGINE_POPUPMENU_NOTYPE;

            GulCString tag = uTag;
            if (tag.EqualsAsciiIgnoreCase("a"))
            {
                GulString tmp;
                nsCOMPtr <nsIDOMHTMLAnchorElement> anchor =
                    do_QueryInterface(node);
                anchor->GetHref (tmp);

                GulCString href = tmp;

                if (href.StartsWith ("mailto:"))
                {
                    href.Cut (0, 7);
                    ctx_data->href = ToNewCString(href);
                    g_strdelimit (ctx_data->href, "?", '\0');
                    flags |= G_WEBENGINE_POPUPMENU_MAILTO;
                }
                else
                {
                    ctx_data->href = ToNewCString(href);
                }

                if (anchor && !tmp.IsEmpty())
                {
                    flags |= G_WEBENGINE_POPUPMENU_LINK;
                    GulCString linkType = tmp;
                    if (linkType.EqualsAsciiIgnoreCase("text/smartbookmark"))
                    {
                        nsCOMPtr<nsIDOMNode> childNode;
                        node->GetFirstChild (getter_AddRefs(childNode));
                        if (childNode)
                        {
                            nsCOMPtr <nsIDOMHTMLImageElement> image =
                                do_QueryInterface(childNode, &rv);
                            if (image)
                            {
                                flags |= G_WEBENGINE_POPUPMENU_IMAGE;
                                GulString img;
                                rv = image->GetSrc (img);
                                if (!NS_FAILED(rv))
                                {
                                    ctx_data->imghref = ToNewCString(img);
                                }
                            }
                        }
                    }
                }
            }
            else if (tag.EqualsAsciiIgnoreCase("option"))
            {
                return G_WEBENGINE_POPUPMENU_NOTYPE;
            }
            else if (tag.EqualsAsciiIgnoreCase("area"))
            {
                nsCOMPtr <nsIDOMHTMLAreaElement> area =
                    do_QueryInterface(node, &rv);
                if (NS_SUCCEEDED(rv) && area)
                {

                    GulString href;
                    rv = area->GetHref (href);
                    if (NS_FAILED(rv))
                        return (G_WEBENGINE_POPUPMENU_NOTYPE);
                    ctx_data->href = ToNewCString(href);
                }
            }
            else if (tag.EqualsAsciiIgnoreCase("input"))
            {
                nsCOMPtr<nsIDOMHTMLInputElement> input = do_QueryInterface (node);
                GulString ginput;
                rv = input->GetType(ginput);
                GulCString gcinput (ginput);
                if (gcinput.EqualsAsciiIgnoreCase("text"))
                {
                    flags |= CONTEXT_INPUT;
                }
            }
            else if (tag.EqualsAsciiIgnoreCase("textarea"))
            {
                flags |= CONTEXT_INPUT;
            }
        }

        nsCOMPtr<nsIDOMNode> parentNode;
        node->GetParentNode (getter_AddRefs(parentNode));
        node = parentNode;
    }
    flags |= G_WEBENGINE_POPUPMENU_DOCUMENT;
    return(flags);
}

nsresult gtk_mozilla_set_profile (const char *aDir, const char *aName)
{

  // what is our profile path?
  static char                   *sProfileDir;
  static char                   *sProfileName;
  static nsProfileDirServiceProvider *sProfileDirServiceProvider;
  static nsIPref                *sPrefs;

  if (sProfileDir) {
    nsMemory::Free(sProfileDir);
    sProfileDir = nsnull;
  }

  if (sProfileName) {
    nsMemory::Free(sProfileName);
    sProfileName = nsnull;
  }

  if (aDir)
    sProfileDir = (char *)nsMemory::Clone(aDir, strlen(aDir) + 1);

  if (aName)
    sProfileName = (char *)nsMemory::Clone(aName, strlen(aName) + 1);

  // initialize profiles
  if (sProfileDir && sProfileName) {
    nsresult rv;
    nsCOMPtr<nsILocalFile> profileDir;
    NS_NewNativeLocalFile(nsDependentCString(sProfileDir), PR_TRUE,
                          getter_AddRefs(profileDir));
    if (!profileDir)
      return NS_ERROR_FAILURE;
    rv = profileDir->AppendNative(nsDependentCString(sProfileName));
    if (NS_FAILED(rv))
      return NS_ERROR_FAILURE;

    nsCOMPtr<nsProfileDirServiceProvider> locProvider;
    NS_NewProfileDirServiceProvider(PR_TRUE, getter_AddRefs(locProvider));
    if (!locProvider)
      return NS_ERROR_FAILURE;
    rv = locProvider->Register();
    if (NS_FAILED(rv))
      return rv;
    rv = locProvider->SetProfileDir(profileDir);
    if (NS_FAILED(rv))
      return rv;
    // Keep a ref so we can shut it down.
    NS_ADDREF(sProfileDirServiceProvider = locProvider);

    // get prefs
    nsCOMPtr<nsIPref> pref;
    pref = do_GetService(NS_PREF_CONTRACTID);
    if (!pref)
      return NS_ERROR_FAILURE;
    sPrefs = pref.get();
    NS_ADDREF(sPrefs);
  }
  return NS_OK;

}
