/** Copyright (C) 2005 Nokia Corporation
* Opera Adapter.
* @file gtkwebengine.c
*/
/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
#include <stdio.h>

#include "../include/gtkwebengine.h"
#include "gtkwebenginemarshal.h"

#define GET_OBJECT_CLASS_TYPE(x) G_OBJECT_CLASS_TYPE(x)


/* class and instance initialization */

static void
gtk_web_engine_class_init(GtkWebEngineClass *klass);

static void
gtk_web_engine_init(GtkWebEngine *embed);


/* globals for this type of widget */

guint gtk_web_engine_signals[GTK_WEBENGINE_LAST] = { 0 };

/* GtkObject + class-related functions */

GType
gtk_web_engine_get_type(void)
{
    static GType web_engine_type = 0;

    if (!web_engine_type)
    {
        static const GTypeInfo web_engine_info =
            {
                sizeof(GtkWebEngineClass),
                NULL, /* base_init */
                NULL, /* base_finalize */
                (GClassInitFunc) gtk_web_engine_class_init,
                NULL, /* class_finalize */
                NULL, /* class_data */
                sizeof(GtkWebEngine),
                0,/* n_preallocs */
                (GInstanceInitFunc)gtk_web_engine_init,
                NULL
            };

        web_engine_type = g_type_register_static (GTK_TYPE_BIN, "GtkWebEngine", &web_engine_info, G_TYPE_FLAG_ABSTRACT);
    }
    return web_engine_type;
}

static void
gtk_web_engine_class_init(GtkWebEngineClass *klass)
{
    /************************************/
    /*          SIGNALS                 */
    /************************************/

    gtk_web_engine_signals[GTK_WEBENGINE_URL_CHANGED_SIGNAL] =
        g_signal_new("url-changed",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, url_changed),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_START_LOADING_SIGNAL] =
        g_signal_new("start-loading",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, start_loading),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_LOADING_PROGRESS_SIGNAL] =
        g_signal_new("loading-progress",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, loading_progress),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__INT_INT,
                     G_TYPE_NONE,
                     2,
                     G_TYPE_INT, G_TYPE_INT);


    gtk_web_engine_signals[GTK_WEBENGINE_FINISHED_LOADING_SIGNAL] =
        g_signal_new("finished-loading",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, finished_loading),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__INT_INT_BOOL,
                     G_TYPE_NONE,
                     3,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_BOOLEAN);

    gtk_web_engine_signals[GTK_WEBENGINE_ENCODING_SIGNAL] =
        g_signal_new("encoding",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, encoding),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__INT,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_INT);


    gtk_web_engine_signals[GTK_WEBENGINE_IMAGE_MODE_CHANGED_SIGNAL]  =
        g_signal_new("image-mode-changed",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, image_mode_changed),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__INT,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_INT);

    gtk_web_engine_signals[GTK_WEBENGINE_SECURITY_MODE_CHANGED_SIGNAL] =
        g_signal_new("security-mode-changed",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, security_mode_changed),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__INT,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_INT);

    /* 	gtk_web_engine_signals[GTK_WEBENGINE_CSS_MODE_CHANGED_SIGNAL] =  */
    /* 		g_signal_new("css-mode-changed", */
    /* 					 G_TYPE_FROM_CLASS(klass), */
    /* 					 G_SIGNAL_RUN_FIRST, */
    /* 					 G_STRUCT_OFFSET(GtkWebEngineClass, css_mode_changed), */
    /* 					 NULL, */
    /* 					 NULL, */
    /* 					 g_cclosure_marshal_VOID__INT, */
    /* 					 G_TYPE_NONE,  */
    /* 					 1, */
    /* 					 G_TYPE_INT); */


    gtk_web_engine_signals[GTK_WEBENGINE_TITLE_CHANGED_SIGNAL] =
        g_signal_new("title",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, title),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_LINK_CLICKED_SIGNAL] =
        g_signal_new("link-clicked",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, link_clicked),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);



    gtk_web_engine_signals[GTK_WEBENGINE_SEARCH_RESET_SIGNAL] = 0; /* FIXME */

    /* 	gtk_web_engine_signals[GTK_WEBENGINE_SCALE_CHANGED_SIGNAL] =  */
    /* 		g_signal_new("scale-changed", */
    /* 					 G_TYPE_FROM_CLASS(klass), */
    /* 					 G_SIGNAL_RUN_FIRST, */
    /* 					 G_STRUCT_OFFSET(GtkWebEngineClass, scale_changed), */
    /* 					 NULL, */
    /* 					 NULL, */
    /* 					 g_cclosure_marshal_VOID__UINT, */
    /* 					 G_TYPE_NONE,  */
    /* 					 1, */
    /* 					 G_TYPE_UINT); */

    gtk_web_engine_signals[GTK_WEBENGINE_MOVE_REQUEST_SIGNAL] = 0; /* FIXME */

    gtk_web_engine_signals[GTK_WEBENGINE_RAISE_REQUEST_SIGNAL] =
        g_signal_new("raise-request",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, raise_request),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    gtk_web_engine_signals[GTK_WEBENGINE_LOWER_REQUEST_SIGNAL] =
        g_signal_new("lower-request",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, lower_request),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);


    gtk_web_engine_signals[GTK_WEBENGINE_ALERT_SIGNAL] =
        g_signal_new("alert",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, alert),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_CONFIRM_SIGNAL] =
        g_signal_new("confirm",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, confirm),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_BOOL__STRING,
                     G_TYPE_BOOLEAN,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_PROMPT_SIGNAL] =
        g_signal_new("prompt",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, prompt),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_STRING__STRING_STRING,
                     G_TYPE_STRING,
                     2,
                     G_TYPE_STRING,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_AUTHENTICATION_REQUIRED_SIGNAL] =
        g_signal_new("authentication-required",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, authentication_required),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__INT_STRING_STRING,
                     G_TYPE_NONE,
                     3,
                     G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);


    gtk_web_engine_signals[GTK_WEBENGINE_UNKNOWN_PROTOCOL_SIGNAL] =
        g_signal_new("unknown-protocol",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, unknown_protocol),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_MAILTO_SIGNAL] =
        g_signal_new("mailto",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, mailto),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);


    gtk_web_engine_signals[GTK_WEBENGINE_DOWNLOAD_REQUEST_SIGNAL] =
        g_signal_new("download-request",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, download_request),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__STRING_STRING_STRING_INT_INT,
                     G_TYPE_NONE,
                     5,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_INT,
                     G_TYPE_INT);


    gtk_web_engine_signals[GTK_WEBENGINE_HAS_BACK_SIGNAL] =
        g_signal_new("has-back",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, has_back),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__BOOLEAN,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_BOOLEAN);

    gtk_web_engine_signals[GTK_WEBENGINE_HAS_FORWARD_SIGNAL] =
        g_signal_new("has-forward",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, has_forward),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__BOOLEAN,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_BOOLEAN);


    gtk_web_engine_signals[GTK_WEBENGINE_EXTERNAL_ACTION_SIGNAL] =
        g_signal_new("external-action",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, external_action),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__STRING,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_STRING);

    gtk_web_engine_signals[GTK_WEBENGINE_ICON_SIGNAL] =
        g_signal_new("icon",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, icon),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__POINTER,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_POINTER);


    gtk_web_engine_signals[GTK_WEBENGINE_WINDOW_REQUEST_SIGNAL] =
        g_signal_new("window-request",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, window_request),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__POINTER_STRING_BOOL_BOOL,
                     G_TYPE_NONE,
                     4,
                     G_TYPE_POINTER,
                     G_TYPE_STRING,
                     G_TYPE_BOOLEAN,
                     G_TYPE_BOOLEAN);

    gtk_web_engine_signals[GTK_WEBENGINE_CLOSE_WINDOW_SIGNAL] =
        g_signal_new("close-window",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, close_window),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    gtk_web_engine_signals[GTK_WEBENGINE_POPUP_MENU_SIGNAL] =
        g_signal_new("webengine-popup-menu",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, popup_menu),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__UINT_INT_INT_STRING_STRING_STRING_STRING,
                     G_TYPE_NONE,
                     7,
                     G_TYPE_UINT,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_STRING);


    gtk_web_engine_signals[GTK_WEBENGINE_FILE_UPLOAD_STARTED_SIGNAL] =
        g_signal_new("file-upload-started",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, file_upload_started),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    gtk_web_engine_signals[GTK_WEBENGINE_FILE_UPLOAD_PROGRESS_SIGNAL] =
        g_signal_new("file-upload-progress",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, file_upload_progress),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__INT_STRING,
                     G_TYPE_NONE,
                     2,
                     G_TYPE_INT, G_TYPE_STRING);


    gtk_web_engine_signals[GTK_WEBENGINE_FILE_UPLOAD_FINISHED_SIGNAL] =
        g_signal_new("file-upload-finished",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, file_upload_finished),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    /*
    	gtk_web_engine_signals[GTK_WEBENGINE_START_IM_COMPOSING_SIGNAL] = 
    		g_signal_new("start-im-composing",
    					 G_TYPE_FROM_CLASS(klass),
    					 G_SIGNAL_RUN_LAST,
    					 G_STRUCT_OFFSET(GtkWebEngineClass, start_im_composing),
    					 NULL,
    					 NULL,
    					 gtk_webengine_marshal_POINTER__INT_STRING_INT_POINTER,
    					 G_TYPE_POINTER, 
    					 4,
    					 G_TYPE_INT,
    					 G_TYPE_STRING,
    					 G_TYPE_INT,
    					 G_TYPE_POINTER);
     
    	gtk_web_engine_signals[GTK_WEBENGINE_STOP_IM_COMPOSING_SIGNAL] = 
    		g_signal_new("stop-im-composing",
    					 G_TYPE_FROM_CLASS(klass),
    					 G_SIGNAL_RUN_LAST,
    					 G_STRUCT_OFFSET(GtkWebEngineClass, stop_im_composing),
    					 NULL,
    					 NULL,
    					 gtk_webengine_marshal_VOID__INT_POINTER_POINTER,
    					 G_TYPE_NONE, 
    					 3,
    					 G_TYPE_INT,
    					 G_TYPE_POINTER,
    					 G_TYPE_POINTER);
    */

    gtk_web_engine_signals[GTK_WEBENGINE_UPLOAD_DIALOG_SIGNAL] =
        g_signal_new("upload-dialog",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, upload_dialog),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_BOOL__STRING_STRING_POINTER,
                     G_TYPE_BOOLEAN,
                     3,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_POINTER);


    gtk_web_engine_signals[GTK_WEBENGINE_MODAL_DIALOG_SIGNAL] =
        g_signal_new("modal-dialog",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, modal_dialog),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_INT__STRING_STRING_INT_INT_INT_INT,
                     G_TYPE_INT,
                     6,
                     G_TYPE_STRING,
                     G_TYPE_STRING,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT);

    gtk_web_engine_signals[GTK_WEBENGINE_TEXT_SELECTED_SIGNAL] =
        g_signal_new("text-selected",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, text_selected),
                     NULL,
                     NULL,
                     gtk_webengine_marshal_VOID__BOOLEAN,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_BOOLEAN);

    gtk_web_engine_signals[GTK_WEBENGINE_SCROLLING_FINISHED_SIGNAL] =
        g_signal_new("scrolling-finished",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, scrolling_finished),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    gtk_web_engine_signals[GTK_WEBENGINE_TEXT_CUT_OR_COPIED_SIGNAL] =
        g_signal_new("text-cut-or-copied",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, text_cut_or_copied),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);

    gtk_web_engine_signals[GTK_WEBENGINE_FRAME_STACKING_CHANGED_SIGNAL] =
        g_signal_new("frame-stacking-changed",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET(GtkWebEngineClass, frame_stacking_changed),
                     NULL,
                     NULL,
                     g_cclosure_marshal_VOID__BOOLEAN,
                     G_TYPE_NONE,
                     1,
                     G_TYPE_BOOLEAN);

    gtk_web_engine_signals[GTK_WEBENGINE_MODAL_CERT_DIALOG_SIGNAL] =
        g_signal_new("modal-certificate-dialog",
                     G_TYPE_FROM_CLASS(klass),
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL,
                     NULL,
                     gtk_webengine_marshal_INT__STRING_INT_INT_INT_INT_INT,
                     G_TYPE_INT,
                     6,
                     G_TYPE_STRING,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT,
                     G_TYPE_INT);


}

static void
gtk_web_engine_init(GtkWebEngine *engine)
{
    GTK_WIDGET_SET_FLAGS (engine, GTK_NO_WINDOW);
}
